@@ -1,11 +1,14 @@
 #include "aot.h"
 
 #include <cstdint>
+#include <cstdlib>
 #include <cstring>
 #include <exception>
 #include <fcntl.h>
 #include <fstream>
 #include <istream>
+#include <optional>
+#include <ostream>
 #include <streambuf>
 #include <sys/mman.h>
 #include <sys/stat.h>
@@ -16,10 +19,14 @@
 #include <cereal/types/unordered_map.hpp>
 #include <cereal/types/vector.hpp>
 
+#include "filesystem.h"
 #include "log.h"
 #include "utils.h"
 
+#define AOT_ELF_SECTION ".btaot"
 static constexpr auto AOT_MAGIC = 0xA07;
+static constexpr auto AOT_SHIM_NAME = "bpftrace-aotrt";
+static constexpr auto AOT_SECDATA_TEMPFILE = ".temp_btaot";
 
 // AOT payload will have this header at the beginning. We don't worry about
 // versioning the header b/c we enforce that an AOT compiled script may only
@@ -63,7 +70,6 @@ uint32_t rs_hash(const std::string &str)
 void serialize_bytecode(const BpfBytecode &bytecode, std::ostream &out)
 {
   cereal::BinaryOutputArchive archive(out);
-  // cereal::JSONOutputArchive archive(out);
   archive(bytecode);
 }
 
@@ -100,11 +106,9 @@ int load_bytecode(BPFtrace &bpftrace, uint8_t *ptr, size_t len)
   return 0;
 }
 
-} // namespace
-
-int generate(const RequiredResources &resources,
-             const BpfBytecode &bytecode,
-             const std::string &out)
+std::optional<std::vector<uint8_t>> generate_btaot_section(
+    const RequiredResources &resources,
+    const BpfBytecode &bytecode)
 {
   // Serialize RuntimeResources
   std::string serialized_metadata;
@@ -117,7 +121,7 @@ int generate(const RequiredResources &resources,
   catch (const std::exception &ex)
   {
     LOG(ERROR) << "Failed to serialize runtime metadata: " << ex.what();
-    return 1;
+    return std::nullopt;
   }
 
   // Serialize bytecode
@@ -131,9 +135,10 @@ int generate(const RequiredResources &resources,
   catch (const std::exception &ex)
   {
     LOG(ERROR) << "Failed to serialize bytecode: " << ex.what();
-    return 1;
+    return std::nullopt;
   }
 
+  // Construct the header
   auto hdr_len = sizeof(Header);
   Header hdr = {
     .magic = AOT_MAGIC,
@@ -146,43 +151,101 @@ int generate(const RequiredResources &resources,
     .bc_len = serialized_bytecode.size(),
   };
 
-  std::ofstream outf(out, std::ios::binary | std::ios::out | std::ios::trunc);
-  if (outf.fail())
-  {
-    LOG(ERROR) << "Failed to open: " << out;
-    return 1;
-  }
+  // Resize the output buffer appropriately
+  std::vector<uint8_t> out;
+  out.resize(sizeof(Header) + hdr.rr_len + hdr.bc_len);
+  uint8_t *p = out.data();
 
   // Write out header
-  outf.write(reinterpret_cast<const char *>(&hdr), sizeof(hdr));
-  if (outf.fail())
-  {
-    LOG(ERROR) << "Failed to write header to: " << out;
-    return 1;
-  }
+  memcpy(p, &hdr, sizeof(Header));
+  p += sizeof(Header);
 
   // Write out metadata
-  outf << serialized_metadata;
-  if (outf.fail())
-  {
-    LOG(ERROR) << "Failed to write metadata to:" << out;
-    return 1;
-  }
+  memcpy(p, serialized_metadata.data(), hdr.rr_len);
+  p += hdr.rr_len;
 
   // Write out bytecode
-  outf << serialized_bytecode;
-  if (outf.fail())
+  memcpy(p, serialized_bytecode.data(), hdr.bc_len);
+  p += hdr.bc_len;
+
+  return out;
+}
+
+// Clones the shim to final destination while also injecting
+// the custom .btaot section.
+bool build_binary(const std_filesystem::path &shim,
+                  const std::string &out,
+                  const std::vector<uint8_t> &section)
+{
+  std::error_code ec;
+  bool ret = false;
+  char cmd[1024];
+  int written;
+
+  // Write out section data in a temporary file
+  std::ofstream secdata(AOT_SECDATA_TEMPFILE, std::ios_base::binary);
+  secdata.write(reinterpret_cast<const char *>(section.data()), section.size());
+  secdata.close();
+
+  // Respect user provided BPFTRACE_OBJCOPY if present
+  std::string objcopy = "objcopy";
+  if (auto c = std::getenv("BPFTRACE_OBJCOPY"))
+    objcopy = c;
+
+  // Resolve objcopy binary to full path
+  auto objcopy_full = find_in_path(objcopy);
+  if (!objcopy_full)
   {
-    LOG(ERROR) << "Failed to write bytecode to:" << out;
+    LOG(ERROR) << "Failed to find " << objcopy << " in $PATH";
+    goto out;
+  }
+
+  written = snprintf(cmd,
+                     sizeof(cmd),
+                     "%s --add-section .btaot=%s %s %s",
+                     objcopy_full->c_str(),
+                     AOT_SECDATA_TEMPFILE,
+                     shim.c_str(),
+                     out.c_str());
+  if (written < 0 || written == sizeof(cmd))
+  {
+    LOG(ERROR) << "Failed to construct objcopy command";
+    goto out;
+  }
+
+  if (std::system(cmd))
+  {
+    LOG(ERROR) << "Failed to execute: " << cmd;
+    goto out;
+  }
+
+  ret = true;
+out:
+  if (!std_filesystem::remove(AOT_SECDATA_TEMPFILE, ec) || ec)
+    LOG(ERROR) << "Failed to remove " << AOT_SECDATA_TEMPFILE << ": " << ec;
+  return ret;
+}
+
+} // namespace
+
+int generate(const RequiredResources &resources,
+             const BpfBytecode &bytecode,
+             const std::string &out)
+{
+  auto section = generate_btaot_section(resources, bytecode);
+  if (!section)
+    return 1;
+
+  auto shim = find_in_path(AOT_SHIM_NAME);
+  if (!shim)
+  {
+    LOG(ERROR) << "Failed to locate " << AOT_SHIM_NAME
+               << " shim binary. Is it in $PATH?";
     return 1;
   }
 
-  outf.flush();
-  if (outf.fail())
-  {
-    LOG(ERROR) << "Failed to flush: " << out;
-    return 1;
-  }
+  if (auto err = build_binary(*shim, out, *section))
+    return err;
 
   return 0;
 }
