@@ -108,5 +108,10 @@ std::vector<std::string> invalid_watchpoint_modes()
       "Watchpoints are not supported on this architecture");
 }
 
+int get_kernel_ptr_width()
+{
+  return 64;
+}
+
 } // namespace arch
 } // namespace bpftrace
