@@ -87,5 +87,21 @@ std::string name()
   return std::string("x86_64");
 }
 
+std::vector<std::string> invalid_watchpoint_modes()
+{
+  // See intel developer manual, Volume 3, section 17.2.4
+  return std::vector<std::string>{
+    "r",
+    "rx",
+    "wx",
+    "rwx",
+  };
+}
+
+int get_kernel_ptr_width()
+{
+  return 64;
+}
+
 } // namespace arch
 } // namespace bpftrace
