@@ -13,8 +13,8 @@ inline bool needMemcpy(const SizedType &stype)
 inline bool shouldBeOnStackAlready(const SizedType &type)
 {
   return type.IsStringTy() || type.IsBufferTy() || type.IsInetTy() ||
-         type.IsUsymTy() || type.IsTupleTy() || type.IsTimestampTy() ||
-         type.IsMacAddressTy() || type.IsCgroupPathTy();
+         type.IsUsymTy() || type.IsUstackTy() || type.IsTupleTy() ||
+         type.IsTimestampTy() || type.IsMacAddressTy() || type.IsCgroupPathTy();
 }
 
 inline bool onStack(const SizedType &type)
