@@ -11,6 +11,7 @@
 #include <llvm/IR/Module.h>
 #include <llvm/Support/raw_os_ostream.h>
 
+#include "ast/dibuilderbpf.h"
 #include "ast/irbuilderbpf.h"
 #include "ast/visitors.h"
 #include "bpftrace.h"
@@ -204,6 +205,8 @@ private:
 
   void createIncDec(Unop &unop);
 
+  Function *createMapLenCallback();
+
   // Return a lambda that has captured-by-value CodegenLLVM's async id state
   // (ie `printf_id_`, `mapped_printf_id_`, etc.).  Running the returned lambda
   // will restore `CodegenLLVM`s async id state back to when this function was
@@ -218,6 +221,8 @@ private:
   std::unique_ptr<Module> module_;
   IRBuilderBPF b_;
 
+  DIBuilderBPF debug_;
+
   const DataLayout &datalayout() const
   {
     return module_->getDataLayout();
