@@ -1,7 +1,10 @@
 #include "bpffeature.h"
 
+#include <bcc/bcc_syms.h>
 #include <bcc/libbpf.h>
 #include <bpf/bpf.h>
+#include <bpf/btf.h>
+#include <bpf/libbpf.h>
 #include <cstddef>
 #include <cstdio>
 #include <fcntl.h>
@@ -19,6 +22,26 @@ namespace bpftrace {
 
 #define ARRAY_SIZE(x) (sizeof(x) / sizeof((x)[0]))
 
+int BPFnofeature::parse(const char* str)
+{
+  for (auto feat : split_string(str, ','))
+  {
+    if (feat == "kprobe_multi")
+    {
+      kprobe_multi_ = true;
+    }
+    else if (feat == "uprobe_multi")
+    {
+      uprobe_multi_ = true;
+    }
+    else
+    {
+      return -1;
+    }
+  }
+  return 0;
+}
+
 static bool try_load_(const char* name,
                       enum libbpf::bpf_prog_type prog_type,
                       std::optional<libbpf::bpf_attach_type> attach_type,
@@ -114,6 +137,25 @@ bool BPFfeature::try_load(enum libbpf::bpf_prog_type prog_type,
                    outfd);
 }
 
+bool BPFfeature::try_load_btf(const void* btf_data, size_t btf_size)
+{
+  constexpr int log_size = 4096;
+  char log_buf[log_size] = {};
+  LIBBPF_OPTS(bpf_btf_load_opts,
+              btf_opts,
+              .log_buf = log_buf,
+              .log_level = 0,
+              .log_size = log_size, );
+
+  int fd = bpf_btf_load(btf_data, btf_size, &btf_opts);
+  if (fd >= 0)
+  {
+    close(fd);
+    return true;
+  }
+  return false;
+}
+
 bool BPFfeature::detect_helper(enum libbpf::bpf_func_id func_id,
                                enum libbpf::bpf_prog_type prog_type)
 {
@@ -227,6 +269,27 @@ bool BPFfeature::has_btf(void)
   return btf_.has_data();
 }
 
+bool BPFfeature::has_btf_func_global()
+{
+  if (has_btf_func_global_.has_value())
+    return *has_btf_func_global_;
+
+  /* static void x(int a) {} */
+  __u32 types[] = {
+    /* int */
+    BTF_TYPE_INT_ENC(1, BTF_INT_SIGNED, 0, 32, 4), /* [1] */
+    /* FUNC_PROTO */                               /* [2] */
+    BTF_TYPE_ENC(0, BTF_INFO_ENC(BTF_KIND_FUNC_PROTO, 0, 1), 0),
+    BTF_PARAM_ENC(7, 1),
+    /* FUNC x BTF_FUNC_GLOBAL */ /* [3] */
+    BTF_TYPE_ENC(5, BTF_INFO_ENC(BTF_KIND_FUNC, 0, BTF_FUNC_GLOBAL), 2),
+  };
+
+  has_btf_func_global_ = std::make_optional<bool>(
+      try_load_btf(types, sizeof(types)));
+  return *has_btf_func_global_;
+}
+
 int BPFfeature::instruction_limit(void)
 {
   if (insns_limit_.has_value())
@@ -356,6 +419,12 @@ bool BPFfeature::has_kprobe_multi()
   if (has_kprobe_multi_.has_value())
     return *has_kprobe_multi_;
 
+  if (no_feature_.kprobe_multi_)
+  {
+    has_kprobe_multi_ = false;
+    return *has_kprobe_multi_;
+  }
+
   const char* sym = "ksys_read";
   DECLARE_LIBBPF_OPTS(bpf_link_create_opts, link_opts);
   int progfd, linkfd = -1;
@@ -372,7 +441,8 @@ bool BPFfeature::has_kprobe_multi()
   load_opts.expected_attach_type = static_cast<enum ::bpf_attach_type>(
       libbpf::BPF_TRACE_KPROBE_MULTI);
 
-  progfd = bpf_prog_load(::BPF_PROG_TYPE_KPROBE,
+  progfd = bpf_prog_load(static_cast<::bpf_prog_type>(
+                             libbpf::BPF_PROG_TYPE_KPROBE),
                          sym,
                          "GPL",
                          reinterpret_cast<struct bpf_insn*>(insns),
@@ -401,6 +471,71 @@ bool BPFfeature::has_kprobe_multi()
   return *has_kprobe_multi_;
 }
 
+bool BPFfeature::has_uprobe_multi()
+{
+  if (has_uprobe_multi_.has_value())
+    return *has_uprobe_multi_;
+
+#if defined(HAVE_LIBBPF_UPROBE_MULTI)
+  if (no_feature_.uprobe_multi_)
+  {
+    has_uprobe_multi_ = false;
+    return *has_uprobe_multi_;
+  }
+
+  LIBBPF_OPTS(bpf_prog_load_opts,
+              load_opts,
+              .expected_attach_type = static_cast<enum ::bpf_attach_type>(
+                  libbpf::BPF_TRACE_UPROBE_MULTI), );
+
+  int err = 0, progfd, linkfd = -1;
+
+  struct bpf_insn insns[] = {
+    BPF_MOV64_IMM(BPF_REG_0, 0),
+    BPF_EXIT_INSN(),
+  };
+
+  progfd = bpf_prog_load(static_cast<::bpf_prog_type>(
+                             libbpf::BPF_PROG_TYPE_KPROBE),
+                         "uprobe_multi",
+                         "GPL",
+                         reinterpret_cast<struct bpf_insn*>(insns),
+                         ARRAY_SIZE(insns),
+                         &load_opts);
+
+  if (progfd >= 0)
+  {
+    LIBBPF_OPTS(bpf_link_create_opts, link_opts);
+    const unsigned long offset = 0;
+
+    link_opts.uprobe_multi.path = "/";
+    link_opts.uprobe_multi.offsets = &offset;
+    link_opts.uprobe_multi.cnt = 1;
+
+    linkfd = bpf_link_create(progfd,
+                             0,
+                             static_cast<enum ::bpf_attach_type>(
+                                 libbpf::BPF_TRACE_UPROBE_MULTI),
+                             &link_opts);
+    err = -errno;
+  }
+
+  has_uprobe_multi_ = linkfd < 0 && err == -EBADF;
+
+  if (linkfd >= 0)
+  {
+    close(linkfd);
+  }
+  if (progfd >= 0)
+  {
+    close(progfd);
+  }
+#else
+  has_uprobe_multi_ = false;
+#endif // HAVE_LIBBPF_UPROBE_MULTI
+  return *has_uprobe_multi_;
+}
+
 bool BPFfeature::has_skb_output(void)
 {
   if (!has_kfunc())
@@ -502,6 +637,7 @@ std::string BPFfeature::report(void)
       << "  skboutput: " << to_str(has_skb_output())
       << "  get_tai_ns: " << to_str(has_helper_ktime_get_tai_ns())
       << "  get_func_ip: " << to_str(has_helper_get_func_ip())
+      << "  jiffies64: " << to_str(has_helper_jiffies64())
 
       << std::endl;
 
@@ -529,6 +665,7 @@ std::string BPFfeature::report(void)
       << "  perf_event: " << to_str(has_prog_perf_event())
       << "  kfunc: " << to_str(has_kfunc())
       << "  kprobe_multi: " << to_str(has_kprobe_multi())
+      << "  uprobe_multi: " << to_str(has_uprobe_multi())
       << "  raw_tp_special: " << to_str(has_raw_tp_special())
       << "  iter: " << to_str(has_iter("task")) << std::endl;
 
