@@ -1,6 +1,7 @@
 #include <arpa/inet.h>
 #include <cassert>
 #include <cerrno>
+#include <cinttypes>
 #include <cstdio>
 #include <cstring>
 #include <ctime>
@@ -29,20 +30,14 @@
 #include <bpf/libbpf.h>
 
 #include "ast/async_event_types.h"
+#include "bpfprogram.h"
 #include "bpftrace.h"
 #include "log.h"
 #include "printf.h"
-#include "relocator.h"
 #include "resolve_cgroupid.h"
 #include "triggers.h"
 #include "utils.h"
 
-namespace libbpf {
-#define __BPF_NAME_FN(x) #x
-const char *bpf_func_name[] = { __BPF_FUNC_MAPPER(__BPF_NAME_FN) };
-#undef __BPF_NAME_FN
-} // namespace libbpf
-
 namespace bpftrace {
 
 DebugLevel bt_debug = DebugLevel::kNone;
@@ -101,7 +96,7 @@ int BPFtrace::add_probe(ast::Probe &p)
       probe.path = "/proc/self/exe";
       probe.attach_point = attach_point->provider + "_trigger";
       probe.type = probetype(attach_point->provider);
-      probe.log_size = log_size_;
+      probe.log_size = config_.get(ConfigKeyInt::log_size);
       probe.orig_name = p.name();
       probe.name = p.name();
       probe.loc = 0;
@@ -156,7 +151,7 @@ int BPFtrace::add_probe(ast::Probe &p)
         Probe probe;
         probe.attach_point = attach_point->func;
         probe.type = probetype(attach_point->provider);
-        probe.log_size = log_size_;
+        probe.log_size = config_.get(ConfigKeyInt::log_size);
         probe.orig_name = p.name();
         probe.name = attach_point->name(attach_point->target,
                                         attach_point->func);
@@ -166,6 +161,63 @@ int BPFtrace::add_probe(ast::Probe &p)
         resources.probes.push_back(probe);
         continue;
       }
+
+      if ((probetype(attach_point->provider) == ProbeType::uprobe ||
+           probetype(attach_point->provider) == ProbeType::uretprobe) &&
+          feature_->has_uprobe_multi() && has_wildcard(attach_point->func) &&
+          !p.need_expansion && attach_funcs.size())
+      {
+        if (!has_wildcard(attach_point->target))
+        {
+          Probe probe;
+          probe.attach_point = attach_point->func;
+          probe.path = attach_point->target;
+          probe.type = probetype(attach_point->provider);
+          probe.log_size = config_.get(ConfigKeyInt::log_size);
+          probe.orig_name = p.name();
+          probe.name = attach_point->name(attach_point->target,
+                                          attach_point->func);
+          probe.index = p.index();
+          probe.funcs = attach_funcs;
+
+          resources.probes.push_back(probe);
+          continue;
+        }
+        else
+        {
+          // If we have a wildcard in the target path, we need to generate one
+          // probe per expanded target.
+          std::unordered_map<std::string, Probe> target_map;
+          for (const auto &func : attach_funcs)
+          {
+            std::string func_id = func;
+            std::string target = erase_prefix(func_id);
+            auto found = target_map.find(target);
+            if (found != target_map.end())
+            {
+              found->second.funcs.push_back(func);
+            }
+            else
+            {
+              Probe probe;
+              probe.attach_point = attach_point->func;
+              probe.path = target;
+              probe.type = probetype(attach_point->provider);
+              probe.log_size = config_.get(ConfigKeyInt::log_size);
+              probe.orig_name = p.name();
+              probe.name = attach_point->name(target, attach_point->func);
+              probe.index = p.index();
+              probe.funcs.push_back(func);
+              target_map.insert({ { target, probe } });
+            }
+          }
+          for (auto &pair : target_map)
+          {
+            resources.probes.push_back(std::move(pair.second));
+          }
+          continue;
+        }
+      }
     }
     else if ((probetype(attach_point->provider) == ProbeType::uprobe ||
               probetype(attach_point->provider) == ProbeType::uretprobe ||
@@ -272,7 +324,7 @@ int BPFtrace::add_probe(ast::Probe &p)
       probe.path = target;
       probe.attach_point = func_id;
       probe.type = probetype(attach_point->provider);
-      probe.log_size = log_size_;
+      probe.log_size = config_.get(ConfigKeyInt::log_size);
       probe.orig_name = p.name();
       probe.ns = attach_point->ns;
       probe.name = attach_point->name(target, func_id);
@@ -323,17 +375,19 @@ int BPFtrace::add_probe(ast::Probe &p)
             resources.probes_using_usym.end() &&
         bcc_elf_is_exe(attach_point->target.c_str()))
     {
+      auto user_symbol_cache_type = config_.get(
+          ConfigKeyUserSymbolCacheType::default_);
       // preload symbol table for executable to make it available even if the
       // binary is not present at symbol resolution time
       // note: this only makes sense with ASLR disabled, since with ASLR offsets
       // might be different
-      if (user_symbol_cache_type_ == UserSymbolCacheType::per_program &&
+      if (user_symbol_cache_type == UserSymbolCacheType::per_program &&
           symbol_table_cache_.find(attach_point->target) ==
               symbol_table_cache_.end())
         symbol_table_cache_[attach_point->target] = get_symbol_table_for_elf(
             attach_point->target);
 
-      if (user_symbol_cache_type_ == UserSymbolCacheType::per_pid)
+      if (user_symbol_cache_type == UserSymbolCacheType::per_pid)
         // preload symbol tables from running processes
         // this allows symbol resolution for processes that are running at probe
         // attach time, but not at symbol resolution time, even with ASLR
@@ -441,7 +495,7 @@ void perf_event_printer(void *cb_cookie, void *data, int size)
   }
   else if (printf_id == asyncactionint(AsyncAction::time))
   {
-    char timestr[64]; // not respecting strlen_ here...
+    char timestr[64]; // not respecting config_.get(ConfigKeyInt::max_strlen)
     time_t t;
     struct tm tmp;
     t = time(NULL);
@@ -482,9 +536,7 @@ void perf_event_printer(void *cb_cookie, void *data, int size)
     auto error_id = helpererror->error_id;
     auto return_value = helpererror->return_value;
     auto &info = bpftrace->resources.helper_error_info[error_id];
-    bpftrace->out_->helper_error(libbpf::bpf_func_name[info.func_id],
-                                 return_value,
-                                 info.loc);
+    bpftrace->out_->helper_error(info.func_id, return_value, info.loc);
     return;
   }
   else if (printf_id == asyncactionint(AsyncAction::watchpoint_attach))
@@ -623,7 +675,9 @@ void perf_event_printer(void *cb_cookie, void *data, int size)
     auto arg_values = bpftrace->get_arg_values(args, arg_data);
 
     std::stringstream buf;
-    cat_file(fmt.format_str(arg_values).c_str(), bpftrace->cat_bytes_max_, buf);
+    cat_file(fmt.format_str(arg_values).c_str(),
+             bpftrace->config_.get(ConfigKeyInt::max_cat_bytes),
+             buf);
     bpftrace->out_->message(MessageType::cat, buf.str(), false);
 
     return;
@@ -714,8 +768,8 @@ std::vector<std::unique_ptr<IPrintable>> BPFtrace::get_arg_values(const std::vec
         auto p = reinterpret_cast<char *>(arg_data + arg.offset);
         arg_values.push_back(std::make_unique<PrintableString>(
             std::string(p, strnlen(p, arg.type.GetSize())),
-            strlen_,
-            str_trunc_trailer_));
+            config_.get(ConfigKeyInt::max_strlen),
+            config_.get(ConfigKeyString::str_trunc_trailer).c_str()));
         break;
       }
       case Type::buffer:
@@ -839,7 +893,7 @@ void perf_event_lost(void *cb_cookie, uint64_t lost)
 
 std::vector<std::unique_ptr<AttachedProbe>> BPFtrace::attach_usdt_probe(
     Probe &probe,
-    std::tuple<uint8_t *, uintptr_t> func,
+    BpfProgram &&program,
     int pid,
     bool file_activation)
 {
@@ -847,8 +901,8 @@ std::vector<std::unique_ptr<AttachedProbe>> BPFtrace::attach_usdt_probe(
 
   if (feature_->has_uprobe_refcnt() || !(file_activation && probe.path.size()))
   {
-    ret.emplace_back(
-        std::make_unique<AttachedProbe>(probe, func, pid, *feature_, *btf_));
+    ret.emplace_back(std::make_unique<AttachedProbe>(
+        probe, std::move(program), pid, *feature_, *btf_));
     return ret;
   }
 
@@ -909,7 +963,7 @@ std::vector<std::unique_ptr<AttachedProbe>> BPFtrace::attach_usdt_probe(
       }
 
       ret.emplace_back(std::make_unique<AttachedProbe>(
-          probe, func, pid_parsed, *feature_, *btf_));
+          probe, std::move(program), pid_parsed, *feature_, *btf_));
       break;
     }
   }
@@ -922,17 +976,9 @@ std::vector<std::unique_ptr<AttachedProbe>> BPFtrace::attach_usdt_probe(
 
 std::vector<std::unique_ptr<AttachedProbe>> BPFtrace::attach_probe(
     Probe &probe,
-    BpfBytecode &bytecode)
+    const BpfBytecode &bytecode)
 {
   std::vector<std::unique_ptr<AttachedProbe>> ret;
-  auto get_section = [](BpfBytecode &bytecode, const std::string &name)
-      -> std::optional<std::tuple<uint8_t *, size_t>> {
-    auto sec = bytecode.find(name);
-    if (sec == bytecode.end())
-      return std::nullopt;
-    return std::make_tuple(sec->second.data(), sec->second.size());
-  };
-
   // use the single-probe program if it exists (as is the case with wildcards
   // and the name builtin, which must be expanded into separate programs per
   // probe), else try to find a the program based on the original probe name
@@ -949,13 +995,17 @@ std::vector<std::unique_ptr<AttachedProbe>> BPFtrace::attach_probe(
                                               probe.index,
                                               usdt_location_idx);
 
-  auto section = get_section(bytecode, name);
-  if (!section)
+  auto program = BpfProgram::CreateFromBytecode(bytecode, name, maps);
+  if (!program)
   {
-    section = get_section(bytecode, orig_name);
+    auto orig_program = BpfProgram::CreateFromBytecode(bytecode,
+                                                       orig_name,
+                                                       maps);
+    if (orig_program)
+      program.emplace(std::move(*orig_program));
   }
 
-  if (!section)
+  if (!program)
   {
     if (probe.name != probe.orig_name)
       LOG(ERROR) << "Code not generated for probe: " << probe.name
@@ -965,18 +1015,14 @@ std::vector<std::unique_ptr<AttachedProbe>> BPFtrace::attach_probe(
     return ret;
   }
 
-  // Make a copy of the bytecode and perform relocations
-  //
-  // We choose not to modify the original bytecode to void keeping
-  // track of state when the same bytecode is attached to multiple probes.
-  std::vector<uint8_t> relocated;
-  relocated.reserve(std::get<1>(*section));
-  memcpy(relocated.data(), std::get<0>(*section), std::get<1>(*section));
-  std::get<0>(*section) = relocated.data();
-  auto relocator = Relocator(*section, *this);
-  if (relocator.relocate())
+  try
   {
-    LOG(ERROR) << "Failed to relocate insns for probe: " << probe.name;
+    program->assemble();
+  }
+  catch (const std::runtime_error &ex)
+  {
+    LOG(ERROR) << "Failed to assemble program for probe: " << probe.name << ", "
+               << ex.what();
     return ret;
   }
 
@@ -986,23 +1032,31 @@ std::vector<std::unique_ptr<AttachedProbe>> BPFtrace::attach_probe(
 
     if (probe.type == ProbeType::usdt)
     {
-      auto aps = attach_usdt_probe(probe, *section, pid, usdt_file_activation_);
+      auto aps = attach_usdt_probe(
+          probe, std::move(*program), pid, usdt_file_activation_);
       for (auto &ap : aps)
         ret.emplace_back(std::move(ap));
 
       return ret;
     }
+    else if (probe.type == ProbeType::uprobe ||
+             probe.type == ProbeType::uretprobe)
+    {
+      ret.emplace_back(std::make_unique<AttachedProbe>(
+          probe, std::move(*program), pid, *feature_, *btf_, safe_mode_));
+      return ret;
+    }
     else if (probe.type == ProbeType::watchpoint ||
              probe.type == ProbeType::asyncwatchpoint)
     {
       ret.emplace_back(std::make_unique<AttachedProbe>(
-          probe, *section, pid, *feature_, *btf_));
+          probe, std::move(*program), pid, *feature_, *btf_));
       return ret;
     }
     else
     {
       ret.emplace_back(std::make_unique<AttachedProbe>(
-          probe, *section, safe_mode_, *feature_, *btf_));
+          probe, std::move(*program), safe_mode_, *feature_, *btf_));
       return ret;
     }
   }
@@ -1062,7 +1116,7 @@ bool attach_reverse(const Probe &p)
 }
 
 int BPFtrace::run_special_probe(std::string name,
-                                BpfBytecode &bytecode,
+                                const BpfBytecode &bytecode,
                                 trigger_fn_t trigger)
 {
   for (auto probe = resources.special_probes.rbegin();
@@ -1071,10 +1125,6 @@ int BPFtrace::run_special_probe(std::string name,
   {
     if ((*probe).attach_point == name)
     {
-      // This is only necessary for uprobe fallback case but it doesn't hurt
-      // to set for raw_tp
-      probe->pid = getpid();
-
       auto aps = attach_probe(*probe, bytecode);
       if (aps.size() != 1)
         return -1;
@@ -1161,17 +1211,18 @@ int BPFtrace::run_iter()
 int BPFtrace::prerun() const
 {
   uint64_t num_probes = this->num_probes();
+  uint64_t max_probes = config_.get(ConfigKeyInt::max_probes);
   if (num_probes == 0)
   {
     if (!bt_quiet)
       std::cout << "No probes to attach" << std::endl;
     return 1;
   }
-  else if (num_probes > max_probes_)
+  else if (num_probes > max_probes)
   {
     LOG(ERROR)
         << "Can't attach to " << num_probes << " probes because it "
-        << "exceeds the current limit of " << max_probes_ << " probes.\n"
+        << "exceeds the current limit of " << max_probes << " probes.\n"
         << "You can increase the limit through the BPFTRACE_MAX_PROBES "
         << "environment variable, but BE CAREFUL since a high number of probes "
         << "attached can cause your system to crash.";
@@ -1195,6 +1246,7 @@ int BPFtrace::run(BpfBytecode bytecode)
     return 1;
 
   bytecode_ = std::move(bytecode);
+  bytecode_.fixupBTF(*feature_);
 
   err = setup_output();
   if (err)
@@ -1351,8 +1403,13 @@ int BPFtrace::setup_perf_events()
   online_cpus_ = cpus.size();
   for (int cpu : cpus)
   {
-    void *reader = bpf_open_perf_buffer(
-        &perf_event_printer, &perf_event_lost, this, -1, cpu, perf_rb_pages_);
+    void *reader = bpf_open_perf_buffer(&perf_event_printer,
+                                        &perf_event_lost,
+                                        this,
+                                        -1,
+                                        cpu,
+                                        config_.get(
+                                            ConfigKeyInt::perf_rb_pages));
     if (reader == nullptr)
     {
       LOG(ERROR) << "Failed to open perf buffer";
@@ -1765,7 +1822,7 @@ int BPFtrace::print_map_hist(IMap &map, uint32_t top, uint32_t div)
     {
       // New key - create a list of buckets for it
       if (map.type_.IsHistTy())
-        values_by_key[key_prefix] = std::vector<uint64_t>(65);
+        values_by_key[key_prefix] = std::vector<uint64_t>(65 * 32);
       else
         values_by_key[key_prefix] = std::vector<uint64_t>(1002);
     }
@@ -1974,7 +2031,7 @@ std::string BPFtrace::get_stack(int64_t stackid,
   return stack.str();
 }
 
-std::string BPFtrace::resolve_uid(uintptr_t addr) const
+std::string BPFtrace::resolve_uid(uint64_t addr) const
 {
   std::string file_name = "/etc/passwd";
   std::string uid = std::to_string(addr);
@@ -2020,7 +2077,7 @@ std::string BPFtrace::resolve_timestamp(uint32_t mode,
     if (!boottime_)
     {
       LOG(ERROR)
-          << "Cannot resolve timestamp due to failed boot time calcuation";
+          << "Cannot resolve timestamp due to failed boot time calculation";
       return "(?)";
     }
     else
@@ -2042,10 +2099,10 @@ std::string BPFtrace::resolve_timestamp(uint32_t mode,
   const auto &raw_fmt = resources.strftime_args[strftime_id];
   uint64_t us = ((basetime->tv_nsec + nsecs) % 1000000000) / 1000;
   char usecs_buf[7];
-  snprintf(usecs_buf, sizeof(usecs_buf), "%06lu", us);
+  snprintf(usecs_buf, sizeof(usecs_buf), "%06" PRIu64, us);
   auto fmt = std::regex_replace(raw_fmt, usec_regex, usecs_buf);
 
-  char timestr[strlen_];
+  char timestr[config_.get(ConfigKeyInt::max_strlen)];
   if (strftime(timestr, sizeof(timestr), fmt.c_str(), &tmp) == 0)
   {
     LOG(ERROR) << "strftime returned 0";
@@ -2059,7 +2116,7 @@ std::string BPFtrace::resolve_buf(char *buf, size_t size)
   return hex_format_buffer(buf, size);
 }
 
-std::string BPFtrace::resolve_ksym(uintptr_t addr, bool show_offset)
+std::string BPFtrace::resolve_ksym(uint64_t addr, bool show_offset)
 {
   struct bcc_symbol ksym;
   std::ostringstream symbol;
@@ -2250,51 +2307,7 @@ std::string BPFtrace::resolve_inet(int af, const uint8_t* inet) const
   return addrstr;
 }
 
-// /proc/sys/kernel/randomize_va_space >= 1 and        // system-wide
-// (/proc/<pid>/personality & ADDR_NO_RNDOMIZE) == 0   // this pid
-// if pid == -1, then only check system-wide setting
-bool BPFtrace::is_aslr_enabled(int pid)
-{
-  std::string randomize_va_space_file = "/proc/sys/kernel/randomize_va_space";
-  std::string personality_file = "/proc/" + std::to_string(pid) +
-                                 "/personality";
-
-  {
-    std::ifstream file(randomize_va_space_file);
-    if (file.fail())
-    {
-      if (bt_verbose)
-        LOG(ERROR) << strerror(errno) << ": " << randomize_va_space_file;
-      // conservatively return true
-      return true;
-    }
-
-    std::string line;
-    if (std::getline(file, line) && std::stoi(line) < 1)
-      return false;
-  }
-
-  if (pid == -1)
-    return true;
-
-  {
-    std::ifstream file(personality_file);
-    if (file.fail())
-    {
-      if (bt_verbose)
-        LOG(ERROR) << strerror(errno) << ": " << personality_file;
-      return true;
-    }
-    std::string line;
-    if (std::getline(file, line) &&
-        ((std::stoi(line) & ADDR_NO_RANDOMIZE) == 0))
-      return true;
-  }
-
-  return false;
-}
-
-std::string BPFtrace::resolve_usym(uintptr_t addr,
+std::string BPFtrace::resolve_usym(uint64_t addr,
                                    int pid,
                                    int probe_id,
                                    bool show_offset,
@@ -2303,6 +2316,8 @@ std::string BPFtrace::resolve_usym(uintptr_t addr,
   struct bcc_symbol usym;
   std::ostringstream symbol;
   void *psyms = nullptr;
+  auto user_symbol_cache_type = config_.get(
+      ConfigKeyUserSymbolCacheType::default_);
 
   if (resolve_user_symbols_)
   {
@@ -2325,7 +2340,7 @@ std::string BPFtrace::resolve_usym(uintptr_t addr,
         pid_exe = probe_full.substr(start, end - start);
       }
     }
-    if (user_symbol_cache_type_ == UserSymbolCacheType::per_program)
+    if (user_symbol_cache_type == UserSymbolCacheType::per_program)
     {
       if (!pid_exe.empty())
       {
@@ -2363,7 +2378,7 @@ std::string BPFtrace::resolve_usym(uintptr_t addr,
         psyms = exe_sym_[pid_exe].second;
       }
     }
-    else if (user_symbol_cache_type_ == UserSymbolCacheType::per_pid)
+    else if (user_symbol_cache_type == UserSymbolCacheType::per_pid)
     {
       // cache user symbols per pid
       if (pid_sym_.find(pid) == pid_sym_.end())
@@ -2386,7 +2401,7 @@ std::string BPFtrace::resolve_usym(uintptr_t addr,
 
   if (psyms && bcc_symcache_resolve(psyms, addr, &usym) == 0)
   {
-    if (demangle_cpp_symbols_)
+    if (config_.get(ConfigKeyBool::cpp_demangle))
       symbol << usym.demangle_name;
     else
       symbol << usym.name;
@@ -2403,7 +2418,7 @@ std::string BPFtrace::resolve_usym(uintptr_t addr,
   }
 
   if (resolve_user_symbols_ &&
-      user_symbol_cache_type_ == UserSymbolCacheType::none)
+      user_symbol_cache_type == UserSymbolCacheType::none)
     bcc_free_symcache(psyms, pid);
 
   return symbol.str();
@@ -2525,13 +2540,22 @@ std::optional<int64_t> BPFtrace::get_int_literal(
   return std::nullopt;
 }
 
+const FuncsModulesMap &BPFtrace::get_traceable_funcs() const
+{
+  if (traceable_funcs_.empty())
+    traceable_funcs_ = parse_traceable_funcs();
+
+  return traceable_funcs_;
+}
+
 bool BPFtrace::is_traceable_func(const std::string &func_name) const
 {
 #ifdef FUZZ
   (void)func_name;
   return true;
 #else
-  return traceable_funcs_.find(func_name) != traceable_funcs_.end();
+  auto &funcs = get_traceable_funcs();
+  return funcs.find(func_name) != funcs.end();
 #endif
 }
 
@@ -2542,9 +2566,9 @@ std::unordered_set<std::string> BPFtrace::get_func_modules(
   (void)func_name;
   return {};
 #else
-  auto mod = traceable_funcs_.find(func_name);
-  return mod != traceable_funcs_.end() ? mod->second
-                                       : std::unordered_set<std::string>();
+  auto &funcs = get_traceable_funcs();
+  auto mod = funcs.find(func_name);
+  return mod != funcs.end() ? mod->second : std::unordered_set<std::string>();
 #endif
 }
 
