@@ -12,9 +12,12 @@
 
 #include "ast/ast.h"
 #include "attached_probe.h"
+#include "bpfbytecode.h"
 #include "bpffeature.h"
+#include "bpfprogram.h"
 #include "btf.h"
 #include "child.h"
+#include "config.h"
 #include "dwarf_parser.h"
 #include "map.h"
 #include "mapmanager.h"
@@ -89,17 +92,17 @@ private:
   std::string msg_;
 };
 
-using BpfBytecode = std::unordered_map<std::string, std::vector<uint8_t>>;
-
 class BPFtrace
 {
 public:
-  BPFtrace(std::unique_ptr<Output> o = std::make_unique<TextOutput>(std::cout))
-      : traceable_funcs_(get_traceable_funcs()),
-        out_(std::move(o)),
-        feature_(std::make_unique<BPFfeature>()),
+  BPFtrace(std::unique_ptr<Output> o = std::make_unique<TextOutput>(std::cout),
+           BPFnofeature no_feature = BPFnofeature(),
+           Config config = Config())
+      : out_(std::move(o)),
+        feature_(std::make_unique<BPFfeature>(no_feature)),
         probe_matcher_(std::make_unique<ProbeMatcher>(this)),
-        ncpus_(get_possible_cpus().size())
+        ncpus_(get_possible_cpus().size()),
+        config_(config)
   {
   }
   virtual ~BPFtrace();
@@ -112,7 +115,7 @@ public:
   int run(BpfBytecode bytecode);
   std::vector<std::unique_ptr<AttachedProbe>> attach_probe(
       Probe &probe,
-      BpfBytecode &bytecode);
+      const BpfBytecode &bytecode);
   int run_iter();
   int print_maps();
   int clear_map(IMap &map);
@@ -125,14 +128,14 @@ public:
                         StackType stack_type,
                         int indent = 0);
   std::string resolve_buf(char *buf, size_t size);
-  std::string resolve_ksym(uintptr_t addr, bool show_offset=false);
-  std::string resolve_usym(uintptr_t addr,
+  std::string resolve_ksym(uint64_t addr, bool show_offset = false);
+  std::string resolve_usym(uint64_t addr,
                            int pid,
                            int probe_id,
                            bool show_offset = false,
                            bool show_module = false);
-  std::string resolve_inet(int af, const uint8_t* inet) const;
-  std::string resolve_uid(uintptr_t addr) const;
+  std::string resolve_inet(int af, const uint8_t *inet) const;
+  std::string resolve_uid(uint64_t addr) const;
   std::string resolve_timestamp(uint32_t mode,
                                 uint32_t strftime_id,
                                 uint64_t nsecs);
@@ -151,7 +154,6 @@ public:
   std::string get_param(size_t index, bool is_str) const;
   size_t num_params() const;
   void request_finalize();
-  bool is_aslr_enabled(int pid);
   std::string get_string_literal(const ast::Expression *expr) const;
   std::optional<int64_t> get_int_literal(const ast::Expression *expr) const;
   std::optional<std::string> get_watchpoint_binary_path() const;
@@ -180,8 +182,7 @@ public:
   std::map<std::string, std::string> macros_;
   std::map<std::string, uint64_t> enums_;
   std::map<libbpf::bpf_func_id, location> helper_use_loc_;
-  // mapping traceable functions to modules (or "vmlinux") that they appear in
-  FuncsModulesMap traceable_funcs_;
+  const FuncsModulesMap &get_traceable_funcs() const;
   KConfig kconfig;
   std::vector<std::unique_ptr<AttachedProbe>> attached_probes_;
 
@@ -192,29 +193,13 @@ public:
   std::unique_ptr<Output> out_;
   std::unique_ptr<BPFfeature> feature_;
 
-  uint64_t strlen_ = 64;
-  const char *str_trunc_trailer_ = "..";
-  uint64_t mapmax_ = 4096;
-  size_t cat_bytes_max_ = 10240;
-  uint64_t max_probes_ = 512;
-  uint64_t max_programs_ = 512;
-  uint64_t log_size_ = 1000000;
-  uint64_t perf_rb_pages_ = 64;
-  uint64_t max_type_res_iterations = 0;
-  bool demangle_cpp_symbols_ = true;
   bool resolve_user_symbols_ = true;
-  enum class UserSymbolCacheType
-  {
-    per_pid,
-    per_program,
-    none,
-  } user_symbol_cache_type_;
   bool safe_mode_ = true;
   bool has_usdt_ = false;
   bool usdt_file_activation_ = false;
   int helper_check_level_ = 0;
-  uint64_t ast_max_nodes_ = 0; // Maximum AST nodes allowed for fuzzing
-  std::optional<StackMode> stack_mode_;
+  uint64_t max_ast_nodes_ = 0; // Maximum AST nodes allowed for fuzzing
+  bool debug_output_ = false;
   std::optional<struct timespec> boottime_;
   std::optional<struct timespec> delta_taitime_;
   static constexpr uint32_t rb_loss_cnt_key_ = 0;
@@ -237,10 +222,11 @@ public:
   }
   int ncpus_;
   int online_cpus_;
+  Config config_;
 
 private:
   int run_special_probe(std::string name,
-                        BpfBytecode &bytecode,
+                        const BpfBytecode &bytecode,
                         void (*trigger)(void));
   void* ksyms_{nullptr};
   // note: exe_sym_ is used when layout is same for all instances of program
@@ -254,7 +240,7 @@ private:
 
   std::vector<std::unique_ptr<AttachedProbe>> attach_usdt_probe(
       Probe &probe,
-      std::tuple<uint8_t *, uintptr_t> func,
+      BpfProgram &&program,
       int pid,
       bool file_activation);
   int setup_output();
@@ -285,6 +271,11 @@ private:
   struct ring_buffer *ringbuf_ = nullptr;
   uint64_t ringbuf_loss_count_ = 0;
 
+  // Mapping traceable functions to modules (or "vmlinux") they appear in.
+  // Needs to be mutable to allow lazy loading of the mapping from const lookup
+  // functions.
+  mutable FuncsModulesMap traceable_funcs_;
+
   std::unordered_map<std::string, std::unique_ptr<Dwarf>> dwarves_;
 };
 
