@@ -17,6 +17,7 @@
 #include "ast/pass_manager.h"
 
 #include "ast/passes/codegen_llvm.h"
+#include "ast/passes/config_analyser.h"
 #include "ast/passes/field_analyser.h"
 #include "ast/passes/node_counter.h"
 #include "ast/passes/portability_analyser.h"
@@ -29,6 +30,7 @@
 #include "build_info.h"
 #include "child.h"
 #include "clang_parser.h"
+#include "config.h"
 #include "driver.h"
 #include "lockdown.h"
 #include "log.h"
@@ -75,6 +77,7 @@ enum Options
   INCLUDE,
   EMIT_ELF,
   EMIT_LLVM,
+  NO_FEATURE,
 };
 } // namespace
 
@@ -116,20 +119,23 @@ void usage()
   std::cerr << "    --emit-llvm FILE        write LLVM IR to FILE.original.ll and FILE.optimized.ll" << std::endl;
   std::cerr << std::endl;
   std::cerr << "ENVIRONMENT:" << std::endl;
-  std::cerr << "    BPFTRACE_STRLEN             [default: 64] bytes on BPF stack per str()" << std::endl;
-  std::cerr << "    BPFTRACE_NO_CPP_DEMANGLE    [default: 0] disable C++ symbol demangling" << std::endl;
-  std::cerr << "    BPFTRACE_MAP_KEYS_MAX       [default: 4096] max keys in a map" << std::endl;
-  std::cerr << "    BPFTRACE_CAT_BYTES_MAX      [default: 10k] maximum bytes read by cat builtin" << std::endl;
-  std::cerr << "    BPFTRACE_MAX_PROBES         [default: 512] max number of probes" << std::endl;
-  std::cerr << "    BPFTRACE_MAX_BPF_PROGS      [default: 512] max number of generated BPF programs" << std::endl;
-  std::cerr << "    BPFTRACE_LOG_SIZE           [default: 1000000] log size in bytes" << std::endl;
-  std::cerr << "    BPFTRACE_PERF_RB_PAGES      [default: 64] pages per CPU to allocate for ring buffer" << std::endl;
-  std::cerr << "    BPFTRACE_NO_USER_SYMBOLS    [default: 0] disable user symbol resolution" << std::endl;
-  std::cerr << "    BPFTRACE_CACHE_USER_SYMBOLS [default: auto] enable user symbol cache" << std::endl;
-  std::cerr << "    BPFTRACE_VMLINUX            [default: none] vmlinux path used for kernel symbol resolution" << std::endl;
-  std::cerr << "    BPFTRACE_BTF                [default: none] BTF file" << std::endl;
-  std::cerr << "    BPFTRACE_STR_TRUNC_TRAILER  [default: '..'] string truncation trailer" << std::endl;
-  std::cerr << "    BPFTRACE_STACK_MODE         [default: bpftrace] Output format for ustack and kstack builtins" << std::endl;
+  std::cerr << "    BPFTRACE_BTF                      [default: none] BTF file" << std::endl;
+  std::cerr << "    BPFTRACE_CACHE_USER_SYMBOLS       [default: auto] enable user symbol cache" << std::endl;
+  std::cerr << "    BPFTRACE_CPP_DEMANGLE             [default: 1] enable C++ symbol demangling" << std::endl;
+  std::cerr << "    BPFTRACE_DEBUG_OUTPUT             [default: 0] enable bpftrace's internal debugging outputs" << std::endl;
+  std::cerr << "    BPFTRACE_KERNEL_BUILD             [default: /lib/modules/$(uname -r)] kernel build directory" << std::endl;
+  std::cerr << "    BPFTRACE_KERNEL_SOURCE            [default: /lib/modules/$(uname -r)] kernel headers directory" << std::endl;
+  std::cerr << "    BPFTRACE_LOG_SIZE                 [default: 1000000] log size in bytes" << std::endl;
+  std::cerr << "    BPFTRACE_MAX_BPF_PROGS            [default: 512] max number of generated BPF programs" << std::endl;
+  std::cerr << "    BPFTRACE_MAX_CAT_BYTES            [default: 10k] maximum bytes read by cat builtin" << std::endl;
+  std::cerr << "    BPFTRACE_MAX_MAP_KEYS             [default: 4096] max keys in a map" << std::endl;
+  std::cerr << "    BPFTRACE_MAX_PROBES               [default: 512] max number of probes" << std::endl;
+  std::cerr << "    BPFTRACE_MAX_STRLEN               [default: 64] bytes on BPF stack per str()" << std::endl;
+  std::cerr << "    BPFTRACE_MAX_TYPE_RES_ITERATIONS  [default: 0] number of levels of nested field accesses for tracepoint args" << std::endl;
+  std::cerr << "    BPFTRACE_PERF_RB_PAGES            [default: 64] pages per CPU to allocate for ring buffer" << std::endl;
+  std::cerr << "    BPFTRACE_STACK_MODE               [default: bpftrace] Output format for ustack and kstack builtins" << std::endl;
+  std::cerr << "    BPFTRACE_STR_TRUNC_TRAILER        [default: '..'] string truncation trailer" << std::endl;
+  std::cerr << "    BPFTRACE_VMLINUX                  [default: none] vmlinux path used for kernel symbol resolution" << std::endl;
   std::cerr << std::endl;
   std::cerr << "EXAMPLES:" << std::endl;
   std::cerr << "bpftrace -l '*sleep*'" << std::endl;
@@ -165,7 +171,7 @@ bool is_root()
     return true;
 }
 
-static void info()
+static void info(BPFnofeature no_feature)
 {
   struct utsname utsname;
   uname(&utsname);
@@ -179,7 +185,7 @@ static void info()
   std::cerr << BuildInfo::report();
 
   std::cerr << std::endl;
-  std::cerr << BPFfeature().report();
+  std::cerr << BPFfeature(no_feature).report();
 }
 
 static std::optional<struct timespec> get_delta_with_boottime(int clock_type)
@@ -256,25 +262,33 @@ static std::optional<struct timespec> get_delta_taitime()
   return get_delta_with_boottime(CLOCK_TAI);
 }
 
-[[nodiscard]] static bool parse_env(BPFtrace& bpftrace, bool& verify_llvm_ir)
+[[nodiscard]] static bool parse_env(BPFtrace& bpftrace)
 {
-  if (!get_uint64_env_var("BPFTRACE_STRLEN", bpftrace.strlen_))
+  ConfigSetter config_setter(bpftrace.config_, ConfigSource::env_var);
+  if (!get_uint64_env_var("BPFTRACE_MAX_STRLEN", [&](uint64_t x) {
+        config_setter.set(ConfigKeyInt::max_strlen, x);
+      }))
     return false;
 
-  if (const char* env_p = std::getenv("BPFTRACE_STR_TRUNC_TRAILER"))
-    bpftrace.str_trunc_trailer_ = env_p;
+  if (!get_uint64_env_var("BPFTRACE_STRLEN", [&](uint64_t x) {
+        LOG(WARNING) << "BPFTRACE_STRLEN is deprecated. Use "
+                        "BPFTRACE_MAX_STRLEN instead.";
+        config_setter.set(ConfigKeyInt::max_strlen, x);
+      }))
+    return false;
 
   // in practice, the largest buffer I've seen fit into the BPF stack was 240
   // bytes. I've set the bar lower, in case your program has a deeper stack than
   // the one from my tests, in the hope that you'll get this instructive error
   // instead of getting the BPF verifier's error.
-  if (bpftrace.strlen_ > 200)
+  uint64_t max_strlen = bpftrace.config_.get(ConfigKeyInt::max_strlen);
+  if (max_strlen > 200)
   {
     // the verifier errors you would encounter when attempting larger
     // allocations would be: >240=  <Looks like the BPF stack limit of 512 bytes
     // is exceeded. Please move large on stack variables into BPF per-cpu array
     // map.> ~1024= <A call to built-in function 'memset' is not supported.>
-    LOG(ERROR) << "'BPFTRACE_STRLEN' " << bpftrace.strlen_
+    LOG(ERROR) << "'BPFTRACE_MAX_STRLEN' " << max_strlen
                << " exceeds the current maximum of 200 bytes.\n"
                << "This limitation is because strings are currently stored on "
                   "the 512 byte BPF stack.\n"
@@ -283,108 +297,92 @@ static std::optional<struct timespec> get_delta_taitime()
     return false;
   }
 
-  if (!get_bool_env_var("BPFTRACE_NO_CPP_DEMANGLE",
-                        bpftrace.demangle_cpp_symbols_,
-                        true))
+  if (const char* env_p = std::getenv("BPFTRACE_STR_TRUNC_TRAILER"))
+    config_setter.set(ConfigKeyString::str_trunc_trailer, std::string(env_p));
+
+  if (!get_bool_env_var("BPFTRACE_CPP_DEMANGLE", [&](bool x) {
+        config_setter.set(ConfigKeyBool::cpp_demangle, x);
+      }))
     return false;
 
-  if (!get_uint64_env_var("BPFTRACE_MAP_KEYS_MAX", bpftrace.mapmax_))
+  if (!get_bool_env_var("BPFTRACE_DEBUG_OUTPUT",
+                        [&](bool x) { bpftrace.debug_output_ = x; }))
     return false;
 
-  if (!get_uint64_env_var("BPFTRACE_MAX_PROBES", bpftrace.max_probes_))
+  if (!get_uint64_env_var("BPFTRACE_MAX_MAP_KEYS", [&](uint64_t x) {
+        config_setter.set(ConfigKeyInt::max_map_keys, x);
+      }))
     return false;
 
-  if (!get_uint64_env_var("BPFTRACE_MAX_BPF_PROGS", bpftrace.max_programs_))
+  if (!get_uint64_env_var("BPFTRACE_MAX_PROBES", [&](uint64_t x) {
+        config_setter.set(ConfigKeyInt::max_probes, x);
+      }))
     return false;
 
-  if (!get_uint64_env_var("BPFTRACE_LOG_SIZE", bpftrace.log_size_))
+  if (!get_uint64_env_var("BPFTRACE_MAX_BPF_PROGS", [&](uint64_t x) {
+        config_setter.set(ConfigKeyInt::max_bpf_progs, x);
+      }))
     return false;
 
-  if (!get_uint64_env_var("BPFTRACE_PERF_RB_PAGES", bpftrace.perf_rb_pages_))
+  if (!get_uint64_env_var("BPFTRACE_LOG_SIZE", [&](uint64_t x) {
+        config_setter.set(ConfigKeyInt::log_size, x);
+      }))
     return false;
 
-  if (!get_uint64_env_var("BPFTRACE_MAX_TYPE_RES_ITERATIONS",
-                          bpftrace.max_type_res_iterations))
+  if (!get_uint64_env_var("BPFTRACE_PERF_RB_PAGES", [&](uint64_t x) {
+        config_setter.set(ConfigKeyInt::perf_rb_pages, x);
+      }))
     return false;
 
-  if (const char* env_p = std::getenv("BPFTRACE_CAT_BYTES_MAX"))
-  {
-    uint64_t proposed;
-    std::istringstream stringstream(env_p);
-    if (!(stringstream >> proposed))
-    {
-      LOG(ERROR) << "Env var 'BPFTRACE_CAT_BYTES_MAX' did not contain a valid "
-                    "uint64_t, or was zero-valued.";
-      return false;
-    }
-    bpftrace.cat_bytes_max_ = proposed;
-  }
+  if (!get_uint64_env_var("BPFTRACE_MAX_TYPE_RES_ITERATIONS", [&](uint64_t x) {
+        config_setter.set(ConfigKeyInt::max_type_res_iterations, x);
+      }))
+    return false;
+
+  if (!get_uint64_env_var("BPFTRACE_MAX_CAT_BYTES", [&](uint64_t x) {
+        config_setter.set(ConfigKeyInt::max_cat_bytes, x);
+      }))
+    return false;
 
-  // by default, cache user symbols per program if ASLR is disabled on system
-  // or `-c` option is given, otherwise cache per PID
-  auto default_usym_cache_type =
-      (!bpftrace.cmd_.empty() || !bpftrace.is_aslr_enabled(-1))
-          ? BPFtrace::UserSymbolCacheType::per_program
-          : BPFtrace::UserSymbolCacheType::per_pid;
   if (const char* env_p = std::getenv("BPFTRACE_CACHE_USER_SYMBOLS"))
   {
-    std::string s(env_p);
-    // Note: options 0 and 1 are for compatibility with older versions of
-    // bpftrace
-    if (s == "PER_PID")
-    {
-      bpftrace.user_symbol_cache_type_ = BPFtrace::UserSymbolCacheType::per_pid;
-    }
-    else if (s == "PER_PROGRAM")
-    {
-      bpftrace.user_symbol_cache_type_ =
-          BPFtrace::UserSymbolCacheType::per_program;
-    }
-    else if (s == "1")
-    {
-      bpftrace.user_symbol_cache_type_ = default_usym_cache_type;
-    }
-    else if (s == "NONE" || s == "0")
-    {
-      bpftrace.user_symbol_cache_type_ = BPFtrace::UserSymbolCacheType::none;
-    }
-    else
-    {
-      LOG(ERROR)
-          << "Env var 'BPFTRACE_CACHE_USER_SYMBOLS' did not contain a valid "
-             "value: valid values are PER_PID, PER_PROGRAM, and NONE.";
+    const std::string s(env_p);
+    if (!config_setter.set_user_symbol_cache_type(s))
       return false;
-    }
-  }
-  else
-  {
-    bpftrace.user_symbol_cache_type_ = default_usym_cache_type;
   }
 
-  uint64_t node_max = std::numeric_limits<uint64_t>::max();
-  if (!get_uint64_env_var("BPFTRACE_NODE_MAX", node_max))
-    return false;
-
-  bpftrace.ast_max_nodes_ = node_max;
-
-  if (!get_bool_env_var("BPFTRACE_VERIFY_LLVM_IR", verify_llvm_ir))
+  bpftrace.max_ast_nodes_ = std::numeric_limits<uint64_t>::max();
+  if (!get_uint64_env_var("BPFTRACE_MAX_AST_NODES",
+                          [&](uint64_t x) { bpftrace.max_ast_nodes_ = x; }))
     return false;
 
   if (const char* stack_mode = std::getenv("BPFTRACE_STACK_MODE"))
   {
-    auto found = STACK_MODE_MAP.find(stack_mode);
-    if (found != STACK_MODE_MAP.end())
-    {
-      bpftrace.stack_mode_ = found->second;
-    }
-    else
-    {
-      LOG(ERROR) << "Env var 'BPFTRACE_STACK_MODE' did not contain a valid "
-                    "StackMode: "
-                 << stack_mode;
-    }
+    if (!config_setter.set_stack_mode(stack_mode))
+      return false;
   }
 
+  if (!get_bool_env_var("BPFTRACE_NO_CPP_DEMANGLE", [&](bool x) {
+        LOG(WARNING) << "BPFTRACE_NO_CPP_DEMANGLE is deprecated. Use "
+                        "BPFTRACE_CPP_DEMANGLE=0 instead.";
+        config_setter.set(ConfigKeyBool::cpp_demangle, !x);
+      }))
+    return false;
+
+  if (!get_uint64_env_var("BPFTRACE_CAT_BYTES_MAX", [&](uint64_t x) {
+        LOG(WARNING) << "BPFTRACE_CAT_BYTES_MAX is deprecated. Use "
+                        "BPFTRACE_MAX_CAT_BYTES instead.";
+        config_setter.set(ConfigKeyInt::max_cat_bytes, x);
+      }))
+    return false;
+
+  if (!get_uint64_env_var("BPFTRACE_MAP_KEYS_MAX", [&](uint64_t x) {
+        LOG(WARNING) << "BPFTRACE_MAP_KEYS_MAX is deprecated. Use "
+                        "BPFTRACE_MAX_MAP_KEYS instead.";
+        config_setter.set(ConfigKeyInt::max_map_keys, x);
+      }))
+    return false;
+
   return true;
 }
 
@@ -461,6 +459,7 @@ static std::optional<struct timespec> get_delta_taitime()
 ast::PassManager CreateDynamicPM()
 {
   ast::PassManager pm;
+  pm.AddPass(ast::CreateConfigPass());
   pm.AddPass(ast::CreateSemanticPass());
   pm.AddPass(ast::CreateCounterPass());
   pm.AddPass(ast::CreateResourcePass());
@@ -495,6 +494,7 @@ struct Args
   std::string output_elf;
   std::string output_llvm;
   std::string aot;
+  BPFnofeature no_feature;
   OutputBufferConfig obc = OutputBufferConfig::UNSET;
   BuildMode build_mode = BuildMode::DYNAMIC;
   std::vector<std::string> include_dirs;
@@ -521,6 +521,7 @@ Args parse_args(int argc, char* argv[])
     option{ "no-warnings", no_argument, nullptr, Options::NO_WARNING },
     option{ "test", required_argument, nullptr, Options::TEST },
     option{ "aot", required_argument, nullptr, Options::AOT },
+    option{ "no-feature", required_argument, nullptr, Options::NO_FEATURE },
     option{ nullptr, 0, nullptr, 0 }, // Must be last
   };
 
@@ -533,7 +534,7 @@ Args parse_args(int argc, char* argv[])
       case Options::INFO: // --info
         if (is_root())
         {
-          info();
+          info(args.no_feature);
           exit(0);
         }
         exit(1);
@@ -560,6 +561,14 @@ Args parse_args(int argc, char* argv[])
         args.aot = optarg;
         args.build_mode = BuildMode::AHEAD_OF_TIME;
         break;
+      case Options::NO_FEATURE: // --no-feature
+        if (args.no_feature.parse(optarg))
+        {
+          LOG(ERROR) << "USAGE: --no-feature can only have values "
+                        "'kprobe_multi,uprobe_multi'.";
+          exit(1);
+        }
+        break;
       case 'o':
         args.output_file = optarg;
         break;
@@ -793,13 +802,13 @@ int main(int argc, char* argv[])
 
   libbpf_set_print(libbpf_print);
 
-  BPFtrace bpftrace(std::move(output));
-  bool verify_llvm_ir = false;
+  Config config = Config(!args.cmd_str.empty(), bt_verbose);
+  BPFtrace bpftrace(std::move(output), args.no_feature, config);
 
   if (!args.cmd_str.empty())
     bpftrace.cmd_ = args.cmd_str;
 
-  if (!parse_env(bpftrace, verify_llvm_ir))
+  if (!parse_env(bpftrace))
     return 1;
 
   bpftrace.usdt_file_activation_ = args.usdt_file_activation;
@@ -991,6 +1000,11 @@ int main(int argc, char* argv[])
     {
       llvm.DumpIR(args.output_llvm + ".original.ll");
     }
+
+    bool verify_llvm_ir = false;
+    if (!get_bool_env_var("BPFTRACE_VERIFY_LLVM_IR",
+                          [&](bool x) { verify_llvm_ir = x; }))
+      return 1;
     if (verify_llvm_ir && !llvm.verify())
     {
       LOG(ERROR) << "Verification of generated LLVM IR failed";
@@ -1016,7 +1030,7 @@ int main(int argc, char* argv[])
       llvm.emit_elf(args.output_elf);
       return 0;
     }
-    bytecode = std::move(llvm.emit());
+    bytecode = llvm.emit();
   }
   catch (const std::system_error& ex)
   {
@@ -1045,7 +1059,7 @@ int main(int argc, char* argv[])
   act.sa_handler = [](int) { BPFtrace::sigusr1_recv = true; };
   sigaction(SIGUSR1, &act, NULL);
 
-  err = bpftrace.run(bytecode);
+  err = bpftrace.run(std::move(bytecode));
   if (err)
     return err;
 
