@@ -32,15 +32,10 @@ static int add_symbol(const char* symname,
 
 /*
  * Finds all matches of search_input in the provided input stream.
- *
- * If `ignore_trailing_module` is true, will ignore trailing kernel module.
- * For example, `[ehci_hcd]` will be ignored in:
- *     ehci_disable_ASE [ehci_hcd]
  */
 std::set<std::string> ProbeMatcher::get_matches_in_stream(
     const std::string& search_input,
     std::istream& symbol_stream,
-    bool ignore_trailing_module,
     bool demangle_symbols,
     const char delim)
 {
@@ -51,11 +46,6 @@ std::set<std::string> ProbeMatcher::get_matches_in_stream(
   std::set<std::string> matches;
   while (std::getline(symbol_stream, line, delim))
   {
-    if (ignore_trailing_module && symbol_has_module(line))
-    {
-      line = strip_symbol_module(line);
-    }
-
     if (!wildcard_match(line, tokens, start_wildcard, end_wildcard))
     {
       if (demangle_symbols)
@@ -119,7 +109,6 @@ std::set<std::string> ProbeMatcher::get_matches_for_probetype(
     bool demangle_symbols)
 {
   std::unique_ptr<std::istream> symbol_stream;
-  bool ignore_trailing_module = false;
 
   switch (probe_type)
   {
@@ -127,7 +116,6 @@ std::set<std::string> ProbeMatcher::get_matches_for_probetype(
     case ProbeType::kretprobe:
     {
       symbol_stream = get_symbols_from_traceable_funcs();
-      ignore_trailing_module = true;
       break;
     }
     case ProbeType::uprobe:
@@ -135,7 +123,7 @@ std::set<std::string> ProbeMatcher::get_matches_for_probetype(
     case ProbeType::watchpoint:
     case ProbeType::asyncwatchpoint:
     {
-      symbol_stream = get_func_symbols_from_file(target);
+      symbol_stream = get_func_symbols_from_file(bpftrace_->pid(), target);
       break;
     }
     case ProbeType::tracepoint:
@@ -173,9 +161,7 @@ std::set<std::string> ProbeMatcher::get_matches_for_probetype(
         symbol_stream = bpftrace_->btf_->get_all_funcs();
       else
       {
-        symbol_stream = get_symbols_from_file(
-            tracefs::available_filter_functions());
-        symbol_stream = adjust_kernel_modules(*symbol_stream);
+        symbol_stream = get_symbols_from_traceable_funcs(true);
       }
       break;
     }
@@ -212,8 +198,9 @@ std::set<std::string> ProbeMatcher::get_matches_for_probetype(
   }
 
   if (symbol_stream)
-    return get_matches_in_stream(
-        search_input, *symbol_stream, ignore_trailing_module, demangle_symbols);
+    return get_matches_in_stream(search_input,
+                                 *symbol_stream,
+                                 demangle_symbols);
   else
     return {};
 }
@@ -232,7 +219,7 @@ std::set<std::string> ProbeMatcher::get_matches_in_set(
     stream_in.append(str + "$");
 
   std::istringstream stream(stream_in);
-  return get_matches_in_stream(search_input, stream, false, false, '$');
+  return get_matches_in_stream(search_input, stream, false, '$');
 }
 
 std::unique_ptr<std::istream> ProbeMatcher::get_symbols_from_file(
@@ -249,18 +236,19 @@ std::unique_ptr<std::istream> ProbeMatcher::get_symbols_from_file(
 }
 
 std::unique_ptr<std::istream> ProbeMatcher::get_symbols_from_traceable_funcs(
-    void) const
+    bool with_modules) const
 {
   std::string funcs;
-  for (auto& func_mod : bpftrace_->traceable_funcs_)
+  for (auto& func_mod : bpftrace_->get_traceable_funcs())
   {
-    if (func_mod.second.empty() || *func_mod.second.begin() == "vmlinux")
+    if (with_modules)
     {
-      funcs += func_mod.first + "\n";
+      for (auto& mod : func_mod.second)
+        funcs += mod + ":" + func_mod.first + "\n";
     }
     else
     {
-      funcs += func_mod.first + " [" + *func_mod.second.begin() + "]\n";
+      funcs += func_mod.first + "\n";
     }
   }
   return std::make_unique<std::istringstream>(funcs);
@@ -281,14 +269,22 @@ std::unique_ptr<std::istream> ProbeMatcher::get_symbols_from_file_safe(
 }
 
 std::unique_ptr<std::istream> ProbeMatcher::get_func_symbols_from_file(
+    int pid,
     const std::string& path) const
 {
   if (path.empty())
     return std::make_unique<std::istringstream>("");
 
   std::vector<std::string> real_paths;
-  if (path.find('*') != std::string::npos)
-    real_paths = resolve_binary_path(path);
+  if (path == "*")
+  {
+    if (pid > 0)
+      real_paths = get_mapped_paths_for_pid(pid);
+    else
+      real_paths = get_mapped_paths_for_running_pids();
+  }
+  else if (path.find('*') != std::string::npos)
+    real_paths = resolve_binary_path(path, pid);
   else
     real_paths.push_back(path);
   struct bcc_symbol_option symbol_option;
@@ -371,24 +367,24 @@ std::unique_ptr<std::istream> ProbeMatcher::get_symbols_from_list(
 
 /*
  * Get list of kernel probe types for the purpose of listing.
- * Ignore return probes.
+ * Ignore return probes and aliases.
  */
 std::unique_ptr<std::istream> ProbeMatcher::kernel_probe_list()
 {
   std::string probes;
   for (auto& p : PROBE_LIST)
   {
+    if (!p.show_in_kernel_list)
+    {
+      continue;
+    }
     if (p.type == ProbeType::kfunc)
     {
       // kfunc must be available
       if (bpftrace_->feature_->has_kfunc())
         probes += p.name + "\n";
     }
-    else if (p.name.find("ret") == std::string::npos &&
-             !is_userspace_probe(p.type) && p.type != ProbeType::interval &&
-             p.type != ProbeType::profile && p.type != ProbeType::watchpoint &&
-             p.type != ProbeType::asyncwatchpoint &&
-             p.type != ProbeType::special)
+    else
     {
       probes += p.name + "\n";
     }
@@ -406,8 +402,10 @@ std::unique_ptr<std::istream> ProbeMatcher::userspace_probe_list()
   std::string probes;
   for (auto& p : PROBE_LIST)
   {
-    if (p.name.find("ret") == std::string::npos && is_userspace_probe(p.type))
+    if (p.show_in_userspace_list)
+    {
       probes += p.name + "\n";
+    }
   }
 
   return std::make_unique<std::istringstream>(probes);
@@ -650,26 +648,6 @@ void ProbeMatcher::list_structs(const std::string& search)
     std::cout << match << std::endl;
 }
 
-// Transform the kernel syntax (function [module]) into bpftrace syntax
-// (module:function).
-std::unique_ptr<std::istream> ProbeMatcher::adjust_kernel_modules(
-    std::istream& symbol_list) const
-{
-  auto new_list = std::make_unique<std::stringstream>();
-  std::string line;
-  while (std::getline(symbol_list, line, '\n'))
-  {
-    if (symbol_has_module(line))
-    {
-      auto sym_mod = split_symbol_module(line);
-      *new_list << sym_mod.second << ":" << sym_mod.first << "\n";
-    }
-    else
-      *new_list << "vmlinux:" << line << "\n";
-  }
-  return new_list;
-}
-
 std::unique_ptr<std::istream> ProbeMatcher::adjust_rawtracepoint(
     std::istream& symbol_list) const
 {
