@@ -1,5 +1,6 @@
 #include <algorithm>
 #include <array>
+#include <climits>
 #include <cmath>
 #include <cstring>
 #include <errno.h>
@@ -20,9 +21,11 @@
 #include <system_error>
 #include <tuple>
 #include <unistd.h>
+#include <unordered_set>
 
 #include "bpftrace.h"
 #include "debugfs.h"
+#include "filesystem.h"
 #include "log.h"
 #include "probe_matcher.h"
 #include "tracefs.h"
@@ -35,16 +38,6 @@
 
 #include <linux/version.h>
 
-#if __has_include(<filesystem>)
-#include <filesystem>
-namespace std_filesystem = std::filesystem;
-#elif __has_include(<experimental/filesystem>)
-#include <experimental/filesystem>
-namespace std_filesystem = std::experimental::filesystem;
-#else
-#error "neither <filesystem> nor <experimental/filesystem> are present"
-#endif
-
 namespace {
 
 std::vector<int> read_cpu_range(std::string path)
@@ -116,6 +109,8 @@ const struct vmlinux_location vmlinux_locs[] = {
   { nullptr, false },
 };
 
+constexpr std::string_view PROC_KHEADERS_PATH = "/sys/kernel/kheaders.tar.xz";
+
 static bool pid_in_different_mountns(int pid);
 static std::vector<std::string>
 resolve_binary_path(const std::string &cmd, const char *env_paths, int pid);
@@ -231,9 +226,11 @@ KConfig::KConfig()
   }
 }
 
-bool get_uint64_env_var(const std::string &str, uint64_t &dest)
+bool get_uint64_env_var(const ::std::string &str,
+                        const std::function<void(uint64_t)> &cb)
 {
-  if (const char* env_p = std::getenv(str.c_str()))
+  uint64_t dest;
+  if (const char *env_p = std::getenv(str.c_str()))
   {
     std::istringstream stringstream(env_p);
     if (!(stringstream >> dest))
@@ -242,19 +239,22 @@ bool get_uint64_env_var(const std::string &str, uint64_t &dest)
                  << "' did not contain a valid uint64_t, or was zero-valued.";
       return false;
     }
+    cb(dest);
   }
   return true;
 }
 
-bool get_bool_env_var(const std::string &str, bool &dest, bool neg)
+bool get_bool_env_var(const ::std::string &str,
+                      const std::function<void(bool)> &cb)
 {
   if (const char *env_p = std::getenv(str.c_str()))
   {
+    bool dest;
     std::string s(env_p);
     if (s == "1")
-      dest = !neg;
+      dest = true;
     else if (s == "0")
-      dest = neg;
+      dest = false;
     else
     {
       LOG(ERROR) << "Env var '" << str
@@ -262,10 +262,30 @@ bool get_bool_env_var(const std::string &str, bool &dest, bool neg)
                     "valid value (0 or 1).";
       return false;
     }
+    cb(dest);
   }
   return true;
 }
 
+std::optional<std_filesystem::path> find_in_path(const std::string &name)
+{
+  std::error_code ec;
+
+  const char *path_env = std::getenv("PATH");
+  if (!path_env)
+    return std::nullopt;
+
+  auto paths = split_string(path_env, ':', true);
+  for (const auto &path : paths)
+  {
+    auto fpath = std_filesystem::path(path) / name;
+    if (std_filesystem::exists(fpath, ec))
+      return fpath;
+  }
+
+  return std::nullopt;
+}
+
 std::string get_pid_exe(const std::string &pid)
 {
   std::error_code ec;
@@ -273,11 +293,18 @@ std::string get_pid_exe(const std::string &pid)
   proc_path /= pid;
   proc_path /= "exe";
 
-  if (!std_filesystem::exists(proc_path, ec) ||
-      !std_filesystem::is_symlink(proc_path, ec))
-    return "";
-
-  return std_filesystem::read_symlink(proc_path).string();
+  try
+  {
+    return std_filesystem::read_symlink(proc_path).string();
+  }
+  catch (const std_filesystem::filesystem_error &e)
+  {
+    auto err = e.code().value();
+    if (err == ENOENT || err == EINVAL)
+      return {};
+    else
+      throw e;
+  }
 }
 
 std::string get_pid_exe(pid_t pid)
@@ -285,6 +312,101 @@ std::string get_pid_exe(pid_t pid)
   return get_pid_exe(std::to_string(pid));
 }
 
+std::string get_proc_maps(const std::string &pid)
+{
+  std::error_code ec;
+  std_filesystem::path proc_path{ "/proc" };
+  proc_path /= pid;
+  proc_path /= "maps";
+
+  if (!std_filesystem::exists(proc_path, ec))
+    return "";
+
+  return proc_path.string();
+}
+
+std::string get_proc_maps(pid_t pid)
+{
+  return get_proc_maps(std::to_string(pid));
+}
+
+std::vector<std::string> get_mapped_paths_for_pid(pid_t pid)
+{
+  static std::map<pid_t, std::vector<std::string>> paths_cache;
+
+  auto it = paths_cache.find(pid);
+  if (it != paths_cache.end())
+  {
+    return it->second;
+  }
+
+  std::vector<std::string> paths;
+
+  // start with the exe
+  std::string pid_exe = get_pid_exe(pid);
+  if (!pid_exe.empty() && pid_exe.find("(deleted)") == std::string::npos)
+    paths.push_back(get_pid_exe(pid));
+
+  // get all the mapped libraries
+  std::string maps_path = get_proc_maps(pid);
+  if (maps_path.empty())
+  {
+    LOG(WARNING) << "Maps path is empty";
+    return paths;
+  }
+
+  std::fstream fs(maps_path, std::ios_base::in);
+  if (!fs.is_open())
+  {
+    LOG(WARNING) << "Unable to open procfs mapfile: " << maps_path;
+    return paths;
+  }
+
+  std::unordered_set<std::string> seen_mappings;
+
+  std::string line;
+  // Example mapping:
+  // 7fc8ee4fa000-7fc8ee4fb000 r--p 00000000 00:1f 27168296 /usr/libc.so.6
+  while (std::getline(fs, line))
+  {
+    char buf[PATH_MAX + 1];
+    buf[0] = '\0';
+    auto res = std::sscanf(line.c_str(), "%*s %*s %*x %*s %*u %[^\n]", buf);
+    // skip [heap], [vdso], and non file paths etc...
+    if (res == 1 && buf[0] == '/')
+    {
+      std::string name = buf;
+      if (name.find("(deleted)") == std::string::npos &&
+          seen_mappings.count(name) == 0)
+      {
+        seen_mappings.emplace(name);
+        paths.push_back(std::move(name));
+      }
+    }
+  }
+
+  paths_cache.emplace(pid, paths);
+  return paths;
+}
+
+std::vector<std::string> get_mapped_paths_for_running_pids()
+{
+  std::unordered_set<std::string> unique_paths;
+  for (auto pid : get_all_running_pids())
+  {
+    for (auto &path : get_mapped_paths_for_pid(pid))
+    {
+      unique_paths.insert(std::move(path));
+    }
+  }
+  std::vector<std::string> paths;
+  for (auto &path : unique_paths)
+  {
+    paths.emplace_back(std::move(path));
+  }
+  return paths;
+}
+
 bool has_wildcard(const std::string &str)
 {
   return str.find("*") != std::string::npos ||
@@ -563,12 +685,11 @@ std::vector<std::pair<std::string, std::string>> get_cgroup_paths(
   // Sort paths lexically by name (with the exception of unified, which always
   // comes first)
   std::sort(result.begin(), result.end(), [](auto &pair1, auto &pair2) {
+    if (pair2.first == "unified")
+      return false;
     if (pair1.first == "unified")
       return true;
-    else if (pair2.first == "unified")
-      return false;
-    else
-      return pair1.first < pair2.first;
+    return pair1.first < pair2.first;
   });
 
   return result;
@@ -581,6 +702,22 @@ bool is_dir(const std::string& path)
   return std_filesystem::is_directory(buf, ec);
 }
 
+bool file_exists_and_ownedby_root(const char *f)
+{
+  struct stat st;
+  if (stat(f, &st) == 0)
+  {
+    if (st.st_uid != 0)
+    {
+      LOG(ERROR) << "header file ownership expected to be root: "
+                 << std::string(f);
+      return false;
+    }
+    return true;
+  }
+  return false;
+}
+
 namespace {
   struct KernelHeaderTmpDir {
     KernelHeaderTmpDir(const std::string& prefix) : path{prefix + "XXXXXX"}
@@ -612,15 +749,19 @@ namespace {
   std::string unpack_kheaders_tar_xz(const struct utsname& utsname)
   {
     std::error_code ec;
+#if defined(__ANDROID__)
+    std_filesystem::path path_prefix{ "/data/local/tmp" };
+#else
     std_filesystem::path path_prefix{ "/tmp" };
-    std_filesystem::path path_kheaders{ "/sys/kernel/kheaders.tar.xz" };
+#endif
+    std_filesystem::path path_kheaders{ PROC_KHEADERS_PATH };
     if (const char* tmpdir = ::getenv("TMPDIR")) {
       path_prefix = tmpdir;
     }
     path_prefix /= "kheaders-";
     std_filesystem::path shared_path{ path_prefix.string() + utsname.release };
 
-    if (std_filesystem::exists(shared_path, ec))
+    if (file_exists_and_ownedby_root(shared_path.c_str()))
     {
       // already unpacked
       return shared_path.string();
@@ -644,7 +785,10 @@ namespace {
 
     KernelHeaderTmpDir tmpdir{path_prefix};
 
-    FILE* tar = ::popen(("tar xf /sys/kernel/kheaders.tar.xz -C " + tmpdir.path).c_str(), "w");
+    FILE *tar = ::popen(("tar xf " + std::string(PROC_KHEADERS_PATH) + " -C " +
+                         tmpdir.path)
+                            .c_str(),
+                        "w");
     if (!tar) {
       return "";
     }
@@ -1054,7 +1198,7 @@ std::string hex_format_buffer(const char *buf,
   return std::string(s);
 }
 
-FuncsModulesMap get_traceable_funcs()
+FuncsModulesMap parse_traceable_funcs()
 {
 #ifdef FUZZ
   return {};
@@ -1242,7 +1386,7 @@ std::optional<std::string> abs_path(const std::string &rel_path)
 
 int64_t min_value(const std::vector<uint8_t> &value, int nvalues)
 {
-  int64_t val, max = 0, retval;
+  int64_t val, max = 0;
   for (int i = 0; i < nvalues; i++)
   {
     val = read_data<int64_t>(value.data() + i * sizeof(int64_t));
@@ -1250,20 +1394,7 @@ int64_t min_value(const std::vector<uint8_t> &value, int nvalues)
       max = val;
   }
 
-  /*
-   * This is a hack really until the code generation for the min() function
-   * is sorted out. The way it is currently implemented doesn't allow >
-   * 32 bit quantities and also means we have to do gymnastics with the return
-   * value owing to the way it is stored (i.e., 0xffffffff - val).
-   */
-  if (max == 0) /* If we have applied the zero() function */
-    retval = max;
-  else if ((0xffffffff - max) <= 0) /* A negative 32 bit value */
-    retval = 0 - (max - 0xffffffff);
-  else
-    retval = 0xffffffff - max; /* A positive 32 bit value */
-
-  return retval;
+  return max;
 }
 
 uint64_t max_value(const std::vector<uint8_t> &value, int nvalues)
@@ -1283,12 +1414,6 @@ bool symbol_has_module(const std::string &symbol)
   return !symbol.empty() && symbol[symbol.size() - 1] == ']';
 }
 
-std::string strip_symbol_module(const std::string &symbol)
-{
-  size_t idx = symbol.rfind(" [");
-  return idx != std::string::npos ? symbol.substr(0, idx) : symbol;
-}
-
 std::pair<std::string, std::string> split_symbol_module(
     const std::string &symbol)
 {
@@ -1357,8 +1482,21 @@ std::map<uintptr_t, elf_symbol, std::greater<>> get_symbol_table_for_elf(
 
 std::vector<int> get_pids_for_program(const std::string &program)
 {
+  std::error_code ec;
+  auto program_abs = std_filesystem::canonical(program, ec);
+  if (ec)
+  {
+    // std::filesystem::canonical will fail if we are attaching to a uprobe that
+    // lives in another filesystem namespace. For example,
+    // uprobe:/proc/12345/root/my_program:function1
+    // This shouldn't be a fatal condition as this function is only used to
+    // attach to all running processes for a given binary, and the above uprobe
+    // is targetting a specific process. So if this happens, just return no
+    // pids. The probe will still attach directly to the targeted process.
+    return {};
+  }
+
   std::vector<int> pids;
-  auto program_abs = std_filesystem::canonical(program);
   for (const auto &process : std_filesystem::directory_iterator("/proc"))
   {
     std::string filename = process.path().filename().string();
@@ -1386,4 +1524,30 @@ std::vector<int> get_all_running_pids()
   return pids;
 }
 
+// BPF verifier rejects programs with names containing certain characters, use
+// this function to replace every character not valid for C identifiers by '_'
+std::string sanitise_bpf_program_name(const std::string &name)
+{
+  std::string sanitised_name = name;
+  std::replace_if(
+      sanitised_name.begin(),
+      sanitised_name.end(),
+      [](char c) { return !isalnum(c) && c != '_'; },
+      '_');
+
+  // Kernel KSYM_NAME_LEN is 128 until 6.1
+  // If we'll exceed the limit, hash the string and cap at 127 (+ null byte).
+  if (sanitised_name.size() > 127)
+  {
+    size_t hash = std::hash<std::string>{}(sanitised_name);
+
+    // std::hash returns size_t, so we reserve 2*sizeof(size_t)+1 characters
+    std::ostringstream os;
+    os << sanitised_name.substr(0, 127 - (2 * sizeof(hash)) - 1) << '_'
+       << std::setfill('0') << std::hex << hash;
+    sanitised_name = os.str();
+  }
+  return sanitised_name;
+}
+
 } // namespace bpftrace
