@@ -222,34 +222,43 @@ TEST(bpftrace, add_probes_wildcard)
       "kprobe:sys_read,kprobe:my_*,kprobe:sys_write{}");
 
   auto bpftrace = get_strict_mock_bpftrace();
-  EXPECT_CALL(*bpftrace->mock_probe_matcher, get_symbols_from_traceable_funcs())
+  bpftrace->feature_ = std::make_unique<MockBPFfeature>(false);
+
+  EXPECT_CALL(*bpftrace->mock_probe_matcher,
+              get_symbols_from_traceable_funcs(false))
       .Times(1);
 
-  if (bpftrace->has_kprobe_multi())
-  {
-    ASSERT_EQ(0, bpftrace->add_probe(*probe));
-    ASSERT_EQ(3U, bpftrace->get_probes().size());
-    ASSERT_EQ(0U, bpftrace->get_special_probes().size());
+  ASSERT_EQ(0, bpftrace->add_probe(*probe));
+  ASSERT_EQ(4U, bpftrace->get_probes().size());
+  ASSERT_EQ(0U, bpftrace->get_special_probes().size());
 
-    std::string probe_orig_name =
-        "kprobe:sys_read,kprobe:my_*,kprobe:sys_write";
-    check_kprobe(bpftrace->get_probes().at(0), "sys_read", probe_orig_name);
-    check_kprobe(bpftrace->get_probes().at(1), "my_*", probe_orig_name);
-    check_kprobe(bpftrace->get_probes().at(2), "sys_write", probe_orig_name);
-  }
-  else
-  {
-    ASSERT_EQ(0, bpftrace->add_probe(*probe));
-    ASSERT_EQ(4U, bpftrace->get_probes().size());
-    ASSERT_EQ(0U, bpftrace->get_special_probes().size());
+  std::string probe_orig_name = "kprobe:sys_read,kprobe:my_*,kprobe:sys_write";
+  check_kprobe(bpftrace->get_probes().at(0), "sys_read", probe_orig_name);
+  check_kprobe(bpftrace->get_probes().at(1), "my_one", probe_orig_name);
+  check_kprobe(bpftrace->get_probes().at(2), "my_two", probe_orig_name);
+  check_kprobe(bpftrace->get_probes().at(3), "sys_write", probe_orig_name);
+}
 
-    std::string probe_orig_name =
-        "kprobe:sys_read,kprobe:my_*,kprobe:sys_write";
-    check_kprobe(bpftrace->get_probes().at(0), "sys_read", probe_orig_name);
-    check_kprobe(bpftrace->get_probes().at(1), "my_one", probe_orig_name);
-    check_kprobe(bpftrace->get_probes().at(2), "my_two", probe_orig_name);
-    check_kprobe(bpftrace->get_probes().at(3), "sys_write", probe_orig_name);
-  }
+TEST(bpftrace, add_probes_wildcard_kprobe_multi)
+{
+  ast::Probe *probe = parse_probe(
+      "kprobe:sys_read,kprobe:my_*,kprobe:sys_write{}");
+
+  auto bpftrace = get_strict_mock_bpftrace();
+  bpftrace->feature_ = std::make_unique<MockBPFfeature>(true);
+
+  EXPECT_CALL(*bpftrace->mock_probe_matcher,
+              get_symbols_from_traceable_funcs(false))
+      .Times(1);
+
+  ASSERT_EQ(0, bpftrace->add_probe(*probe));
+  ASSERT_EQ(3U, bpftrace->get_probes().size());
+  ASSERT_EQ(0U, bpftrace->get_special_probes().size());
+
+  std::string probe_orig_name = "kprobe:sys_read,kprobe:my_*,kprobe:sys_write";
+  check_kprobe(bpftrace->get_probes().at(0), "sys_read", probe_orig_name);
+  check_kprobe(bpftrace->get_probes().at(1), "my_*", probe_orig_name);
+  check_kprobe(bpftrace->get_probes().at(2), "sys_write", probe_orig_name);
 }
 
 TEST(bpftrace, add_probes_wildcard_no_matches)
@@ -258,7 +267,8 @@ TEST(bpftrace, add_probes_wildcard_no_matches)
       "kprobe:sys_read,kprobe:not_here_*,kprobe:sys_write{}");
 
   auto bpftrace = get_strict_mock_bpftrace();
-  EXPECT_CALL(*bpftrace->mock_probe_matcher, get_symbols_from_traceable_funcs())
+  EXPECT_CALL(*bpftrace->mock_probe_matcher,
+              get_symbols_from_traceable_funcs(false))
       .Times(1);
 
   ASSERT_EQ(0, bpftrace->add_probe(*probe));
@@ -283,28 +293,6 @@ TEST(bpftrace, add_probes_kernel_module)
   check_kprobe(bpftrace->get_probes().at(0), "func_in_mod", probe_orig_name);
 }
 
-TEST(bpftrace, add_probes_kernel_module_wildcard)
-{
-  ast::Probe *probe = parse_probe("kprobe:func_in_mo*{}");
-  auto bpftrace = get_strict_mock_bpftrace();
-  EXPECT_CALL(*bpftrace->mock_probe_matcher, get_symbols_from_traceable_funcs())
-      .Times(1);
-
-  ASSERT_EQ(0, bpftrace->add_probe(*probe));
-  ASSERT_EQ(1U, bpftrace->get_probes().size());
-  ASSERT_EQ(0U, bpftrace->get_special_probes().size());
-
-  std::string probe_orig_name = "kprobe:func_in_mo*";
-  if (bpftrace->has_kprobe_multi())
-  {
-    check_kprobe(bpftrace->get_probes().at(0), "func_in_mo*", probe_orig_name);
-  }
-  else
-  {
-    check_kprobe(bpftrace->get_probes().at(0), "func_in_mod", probe_orig_name);
-  }
-}
-
 TEST(bpftrace, add_probes_offset)
 {
   auto offset = 10;
@@ -335,8 +323,10 @@ TEST(bpftrace, add_probes_uprobe_wildcard)
   ast::Probe *probe = parse_probe("uprobe:/bin/sh:*open {}");
 
   auto bpftrace = get_strict_mock_bpftrace();
+  bpftrace->feature_ = std::make_unique<MockBPFfeature>(false);
+
   EXPECT_CALL(*bpftrace->mock_probe_matcher,
-              get_func_symbols_from_file("/bin/sh"))
+              get_func_symbols_from_file(0, "/bin/sh"))
       .Times(1);
 
   ASSERT_EQ(0, bpftrace->add_probe(*probe));
@@ -348,12 +338,32 @@ TEST(bpftrace, add_probes_uprobe_wildcard)
   check_uprobe(bpftrace->get_probes().at(1), "/bin/sh", "second_open", probe_orig_name);
 }
 
+TEST(bpftrace, add_probes_uprobe_wildcard_uprobe_multi)
+{
+  ast::Probe *probe = parse_probe("uprobe:/bin/sh:*open {}");
+
+  auto bpftrace = get_strict_mock_bpftrace();
+  bpftrace->feature_ = std::make_unique<MockBPFfeature>(true);
+
+  EXPECT_CALL(*bpftrace->mock_probe_matcher,
+              get_func_symbols_from_file(0, "/bin/sh"))
+      .Times(1);
+
+  std::string probe_orig_name = "uprobe:/bin/sh:*open";
+  ASSERT_EQ(0, bpftrace->add_probe(*probe));
+  ASSERT_EQ(1U, bpftrace->get_probes().size());
+  ASSERT_EQ(0U, bpftrace->get_special_probes().size());
+
+  check_uprobe(
+      bpftrace->get_probes().at(0), "/bin/sh", "*open", probe_orig_name);
+}
+
 TEST(bpftrace, add_probes_uprobe_wildcard_file)
 {
   ast::Probe *probe = parse_probe("uprobe:/bin/*sh:first_open {}");
   auto bpftrace = get_strict_mock_bpftrace();
   EXPECT_CALL(*bpftrace->mock_probe_matcher,
-              get_func_symbols_from_file("/bin/*sh"))
+              get_func_symbols_from_file(0, "/bin/*sh"))
       .Times(1);
 
   ASSERT_EQ(0, bpftrace->add_probe(*probe));
@@ -367,12 +377,62 @@ TEST(bpftrace, add_probes_uprobe_wildcard_file)
       bpftrace->get_probes().at(1), "/bin/sh", "first_open", probe_orig_name);
 }
 
+TEST(bpftrace, add_probes_uprobe_wildcard_for_target)
+{
+  ast::Probe *probe = parse_probe("uprobe:*:*open {}");
+
+  auto bpftrace = get_strict_mock_bpftrace();
+  bpftrace->feature_ = std::make_unique<MockBPFfeature>(false);
+
+  EXPECT_CALL(*bpftrace->mock_probe_matcher, get_func_symbols_from_file(0, "*"))
+      .Times(1);
+
+  ASSERT_EQ(0, bpftrace->add_probe(*probe));
+  ASSERT_EQ(4U, bpftrace->get_probes().size());
+  ASSERT_EQ(0U, bpftrace->get_special_probes().size());
+
+  std::string probe_orig_name = "uprobe:*:*open";
+  check_uprobe(
+      bpftrace->get_probes().at(0), "/bin/bash", "first_open", probe_orig_name);
+  check_uprobe(
+      bpftrace->get_probes().at(1), "/bin/sh", "first_open", probe_orig_name);
+  check_uprobe(
+      bpftrace->get_probes().at(2), "/bin/sh", "second_open", probe_orig_name);
+  check_uprobe(bpftrace->get_probes().at(3),
+               "/proc/1234/exe",
+               "third_open",
+               probe_orig_name);
+}
+
+TEST(bpftrace, add_probes_uprobe_wildcard_for_target_uprobe_multi)
+{
+  ast::Probe *probe = parse_probe("uprobe:*:*open {}");
+
+  auto bpftrace = get_strict_mock_bpftrace();
+  bpftrace->feature_ = std::make_unique<MockBPFfeature>(true);
+
+  EXPECT_CALL(*bpftrace->mock_probe_matcher, get_func_symbols_from_file(0, "*"))
+      .Times(1);
+
+  ASSERT_EQ(0, bpftrace->add_probe(*probe));
+  ASSERT_EQ(3U, bpftrace->get_probes().size());
+  ASSERT_EQ(0U, bpftrace->get_special_probes().size());
+
+  std::string probe_orig_name = "uprobe:*:*open";
+  check_uprobe(
+      bpftrace->get_probes().at(0), "/proc/1234/exe", "*open", probe_orig_name);
+  check_uprobe(
+      bpftrace->get_probes().at(1), "/bin/sh", "*open", probe_orig_name);
+  check_uprobe(
+      bpftrace->get_probes().at(2), "/bin/bash", "*open", probe_orig_name);
+}
+
 TEST(bpftrace, add_probes_uprobe_wildcard_no_matches)
 {
   ast::Probe *probe = parse_probe("uprobe:/bin/sh:foo* {}");
   auto bpftrace = get_strict_mock_bpftrace();
   EXPECT_CALL(*bpftrace->mock_probe_matcher,
-              get_func_symbols_from_file("/bin/sh"))
+              get_func_symbols_from_file(0, "/bin/sh"))
       .Times(1);
 
   ASSERT_EQ(0, bpftrace->add_probe(*probe));
@@ -428,7 +488,7 @@ TEST(bpftrace, add_probes_uprobe_cpp_symbol)
 
     auto bpftrace = get_strict_mock_bpftrace();
     EXPECT_CALL(*bpftrace->mock_probe_matcher,
-                get_func_symbols_from_file("/bin/sh"))
+                get_func_symbols_from_file(0, "/bin/sh"))
         .Times(1);
 
     ASSERT_EQ(0, bpftrace->add_probe(*probe));
@@ -454,7 +514,7 @@ TEST(bpftrace, add_probes_uprobe_cpp_symbol_full)
 
   auto bpftrace = get_strict_mock_bpftrace();
   EXPECT_CALL(*bpftrace->mock_probe_matcher,
-              get_func_symbols_from_file("/bin/sh"))
+              get_func_symbols_from_file(0, "/bin/sh"))
       .Times(1);
 
   ASSERT_EQ(0, bpftrace->add_probe(*probe));
@@ -471,13 +531,16 @@ TEST(bpftrace, add_probes_uprobe_cpp_symbol_wildcard)
   auto probe = parse_probe("uprobe:/bin/sh:cpp:cpp_man*{}");
 
   auto bpftrace = get_strict_mock_bpftrace();
+  bpftrace->feature_ = std::make_unique<MockBPFfeature>(false);
+
   EXPECT_CALL(*bpftrace->mock_probe_matcher,
-              get_func_symbols_from_file("/bin/sh"))
+              get_func_symbols_from_file(0, "/bin/sh"))
       .Times(1);
 
   ASSERT_EQ(0, bpftrace->add_probe(*probe));
   ASSERT_EQ(4U, bpftrace->get_probes().size());
   ASSERT_EQ(0U, bpftrace->get_special_probes().size());
+
   check_uprobe(bpftrace->get_probes().at(0),
                "/bin/sh:cpp",
                "_Z11cpp_mangledi",
@@ -503,7 +566,7 @@ TEST(bpftrace, add_probes_uprobe_no_demangling)
 
   auto bpftrace = get_strict_mock_bpftrace();
   EXPECT_CALL(*bpftrace->mock_probe_matcher,
-              get_func_symbols_from_file("/bin/sh"))
+              get_func_symbols_from_file(0, "/bin/sh"))
       .Times(0);
 
   ASSERT_EQ(0, bpftrace->add_probe(*probe));
