@@ -13,18 +13,9 @@
 
 #include "child.h"
 #include "childhelper.h"
+#include "filesystem.h"
 #include "utils.h"
 
-#if __has_include(<filesystem>)
-#include <filesystem>
-namespace std_filesystem = std::filesystem;
-#elif __has_include(<experimental/filesystem>)
-#include <experimental/filesystem>
-namespace std_filesystem = std::experimental::filesystem;
-#else
-#error "neither <filesystem> nor <experimental/filesystem> are present"
-#endif
-
 namespace bpftrace {
 namespace test {
 namespace child {
