@@ -11,16 +11,17 @@ using ::testing::StrictMock;
 
 void setup_mock_probe_matcher(MockProbeMatcher &matcher)
 {
-  ON_CALL(matcher, get_symbols_from_traceable_funcs()).WillByDefault([](void) {
-    std::string ksyms = "SyS_read\n"
-                        "sys_read\n"
-                        "sys_write\n"
-                        "my_one\n"
-                        "my_two\n"
-                        "func_in_mod [kernel_mod]\n";
-    auto myval = std::unique_ptr<std::istream>(new std::istringstream(ksyms));
-    return myval;
-  });
+  ON_CALL(matcher, get_symbols_from_traceable_funcs(false))
+      .WillByDefault([](void) {
+        std::string ksyms = "SyS_read\n"
+                            "sys_read\n"
+                            "sys_write\n"
+                            "my_one\n"
+                            "my_two\n";
+        auto myval = std::unique_ptr<std::istream>(
+            new std::istringstream(ksyms));
+        return myval;
+      });
 
   ON_CALL(matcher, get_symbols_from_file(tracefs::available_events()))
       .WillByDefault([](const std::string &) {
@@ -43,16 +44,23 @@ void setup_mock_probe_matcher(MockProbeMatcher &matcher)
                          "/bin/sh:_Z11cpp_mangledv\n"
                          "/bin/sh:_Z18cpp_mangled_suffixv\n";
   std::string bash_usyms = "/bin/bash:first_open\n";
-  ON_CALL(matcher, get_func_symbols_from_file("/bin/sh"))
-      .WillByDefault([sh_usyms](const std::string &) {
+  std::string proc_usyms = "/proc/1234/exe:third_open\n";
+  ON_CALL(matcher, get_func_symbols_from_file(_, "/bin/sh"))
+      .WillByDefault([sh_usyms](int, const std::string &) {
         return std::unique_ptr<std::istream>(new std::istringstream(sh_usyms));
       });
 
-  ON_CALL(matcher, get_func_symbols_from_file("/bin/*sh"))
-      .WillByDefault([sh_usyms, bash_usyms](const std::string &) {
+  ON_CALL(matcher, get_func_symbols_from_file(_, "/bin/*sh"))
+      .WillByDefault([sh_usyms, bash_usyms](int, const std::string &) {
         return std::unique_ptr<std::istream>(
             new std::istringstream(sh_usyms + bash_usyms));
       });
+  ON_CALL(matcher, get_func_symbols_from_file(_, "*"))
+      .WillByDefault(
+          [sh_usyms, bash_usyms, proc_usyms](int, const std::string &) {
+            return std::unique_ptr<std::istream>(
+                new std::istringstream(sh_usyms + bash_usyms + proc_usyms));
+          });
 
   std::string sh_usdts = "/bin/sh:prov1:tp1\n"
                          "/bin/sh:prov1:tp2\n"
