@@ -23,13 +23,14 @@ public:
 #endif
   MOCK_CONST_METHOD1(get_symbols_from_file,
                      std::unique_ptr<std::istream>(const std::string &path));
-  MOCK_CONST_METHOD0(get_symbols_from_traceable_funcs,
-                     std::unique_ptr<std::istream>(void));
+  MOCK_CONST_METHOD1(get_symbols_from_traceable_funcs,
+                     std::unique_ptr<std::istream>(bool with_modules));
   MOCK_CONST_METHOD2(get_symbols_from_usdt,
                      std::unique_ptr<std::istream>(int pid,
                                                    const std::string &target));
-  MOCK_CONST_METHOD1(get_func_symbols_from_file,
-                     std::unique_ptr<std::istream>(const std::string &path));
+  MOCK_CONST_METHOD2(get_func_symbols_from_file,
+                     std::unique_ptr<std::istream>(int pid,
+                                                   const std::string &path));
 #pragma GCC diagnostic pop
 };
 
@@ -87,11 +88,6 @@ public:
     mock_probe_matcher = dynamic_cast<MockProbeMatcher *>(probe_matcher_.get());
   }
 
-  bool has_kprobe_multi(void)
-  {
-    return feature_->has_kprobe_multi();
-  }
-
   MockProbeMatcher *mock_probe_matcher;
 };
 
@@ -113,10 +109,12 @@ public:
     has_d_path_ = std::make_optional<bool>(has_features);
     has_ktime_get_boot_ns_ = std::make_optional<bool>(has_features);
     has_kprobe_multi_ = std::make_optional<bool>(has_features);
+    has_uprobe_multi_ = std::make_optional<bool>(has_features);
     has_skb_output_ = std::make_optional<bool>(has_features);
     map_ringbuf_ = std::make_optional<bool>(has_features);
     has_ktime_get_tai_ns_ = std::make_optional<bool>(has_features);
     has_get_func_ip_ = std::make_optional<bool>(has_features);
+    has_jiffies64_ = std::make_optional<bool>(has_features);
   };
 
   void has_loop(bool has)
