@@ -1,5 +1,6 @@
 #!/usr/bin/python3
 
+import json
 import subprocess
 import signal
 import sys
@@ -11,8 +12,9 @@ from functools import lru_cache
 
 import cmake_vars
 
-BPF_PATH = os.environ["BPFTRACE_RUNTIME_TEST_EXECUTABLE"]
-ATTACH_TIMEOUT = 5
+BPFTRACE_BIN = os.environ["BPFTRACE_RUNTIME_TEST_EXECUTABLE"]
+AOT_BIN = os.environ["BPFTRACE_AOT_RUNTIME_TEST_EXECUTABLE"]
+ATTACH_TIMEOUT = 10
 DEFAULT_TIMEOUT = 5
 
 
@@ -50,7 +52,11 @@ class Runner(object):
 
     @staticmethod
     def failed(status):
-        return status in [Runner.FAIL, Runner.TIMEOUT]
+        return status == Runner.FAIL
+
+    @staticmethod
+    def timeouted(status):
+        return status == Runner.TIMEOUT
 
     @staticmethod
     def skipped(status):
@@ -83,22 +89,22 @@ class Runner(object):
             raise ValueError("Invalid skip reason: %d" % status)
 
     @staticmethod
-    def prepare_bpf_call(test):
-        bpftrace_path = "{}/bpftrace".format(BPF_PATH)
-        bpftrace_aotrt_path = "{}/aot/bpftrace-aotrt".format(BPF_PATH)
+    def prepare_bpf_call(test, nsenter=[]):
+        nsenter_prefix = (" ".join(nsenter) + " ") if len(nsenter) > 0 else ""
 
         if test.run:
-            ret = re.sub("{{BPFTRACE}}", bpftrace_path, test.run)
-            ret = re.sub("{{BPFTRACE_AOTRT}}", bpftrace_aotrt_path, ret)
+            ret = re.sub("{{BPFTRACE}}", BPFTRACE_BIN, test.run)
+            ret = re.sub("{{BPFTRACE_AOTRT}}", AOT_BIN, ret)
 
-            return ret
+            return nsenter_prefix + ret
         else:  # PROG
+            use_json = "-q -f json" if test.expect_mode == "json" else ""
+            cmd = nsenter_prefix + "{} {} -e '{}'".format(BPFTRACE_BIN, use_json, test.prog)
             # We're only reusing PROG-directive tests for AOT tests
             if test.suite == 'aot':
-                return "{} -e '{}' --aot /tmp/tmpprog.btaot && {} /tmp/tmpprog.btaot".format(
-                    bpftrace_path, test.prog, bpftrace_aotrt_path)
+                return cmd + " --aot /tmp/tmpprog.btaot && {} /tmp/tmpprog.btaot".format(AOT_BIN)
             else:
-                return "{} -e '{}'".format(bpftrace_path, test.prog)
+                return cmd
 
     @staticmethod
     def __handler(signum, frame):
@@ -108,7 +114,7 @@ class Runner(object):
     @lru_cache(maxsize=1)
     def __get_bpffeature():
         p = subprocess.Popen(
-            [f"{BPF_PATH}/bpftrace", "--info"],
+            [BPFTRACE_BIN, "--info"],
             stdout=subprocess.PIPE,
             stderr=subprocess.STDOUT,
             start_new_session=True,
@@ -130,12 +136,37 @@ class Runner(object):
         bpffeature["libpath_resolv"] = output.find("bcc library path resolution: yes") != -1
         bpffeature["dwarf"] = output.find("libdw (DWARF support): yes") != -1
         bpffeature["kprobe_multi"] = output.find("kprobe_multi: yes") != -1
+        bpffeature["uprobe_multi"] = output.find("uprobe_multi: yes") != -1
         bpffeature["aot"] = cmake_vars.LIBBCC_BPF_CONTAINS_RUNTIME
         bpffeature["skboutput"] = output.find("skboutput: yes") != -1
         bpffeature["get_tai_ns"] = output.find("get_ktime_ns: yes") != -1
         bpffeature["get_func_ip"] = output.find("get_func_ip: yes") != -1
+        bpffeature["jiffies64"] = output.find("jiffies64: yes") != -1
         return bpffeature
 
+
+    @staticmethod
+    def __wait_for_children(parent_pid, timeout, ps_format, condition):
+        with open(os.devnull, 'w') as dn:
+            waited=0
+            while waited <= timeout:
+                run_cmd = ["ps", "--ppid", str(parent_pid), "--no-headers", "-o", ps_format]
+                children = subprocess.run(run_cmd,
+                                          check=False,
+                                          stdout=subprocess.PIPE,
+                                          stderr=subprocess.PIPE)
+                if children.returncode == 0 and children.stdout:
+                    lines = [line.decode('utf-8') for line in children.stdout.splitlines()]
+                    if (condition(lines)):
+                        return lines
+                else:
+                    print(f"__wait_for_children error: {children.stderr}. Return code: {children.returncode}")
+
+                time.sleep(0.1)
+                waited+=0.1
+        return None
+
+
     @staticmethod
     def run_test(test):
         current_kernel = LooseVersion(os.uname()[2])
@@ -161,7 +192,28 @@ class Runner(object):
         after = None
         after_output = None
         cleanup = None
+        # This is only populated if the NEW_PIDNS directive is set
+        # and is used to enter the newly created pid namespace for all BEFOREs,
+        # the primary RUN or PROG command, and the AFTER.
+        nsenter = []
         bpf_call = "[unknown]"
+
+        def get_pid_ns_cmd(cmd):
+            return nsenter + [os.path.abspath(x) for x in cmd.split()]
+
+        def check_result(output):
+            try:
+                if test.expect_mode == "regex":
+                    return re.search(test.expect, output, re.M)
+                elif test.expect_mode == "file":
+                    # remove leading and trailing empty lines
+                    return output.strip() == open(test.expect).read().strip()
+                else:
+                    return json.loads(output) == json.load(open(test.expect))
+            except Exception as err:
+                print("ERROR in check_result: ", err)
+                return False
+
         try:
             result = None
             timeout = False
@@ -197,41 +249,60 @@ class Runner(object):
                         print(warn("[   SKIP   ] ") + "%s.%s" % (test.suite, test.name))
                         return Runner.SKIP_FEATURE_REQUIREMENT_UNSATISFIED
 
+            if test.new_pidns and not test.befores:
+                raise ValueError("`NEW_PIDNS` requires at least one `BEFORE` directive as something needs to run in the new pid namespace")
+
             if test.befores:
-                for before in test.befores:
-                    before = subprocess.Popen(before.split(),
-                                              start_new_session=True,
-                                              universal_newlines=True,
-                                              stdout=subprocess.PIPE,
-                                              stderr=subprocess.STDOUT)
-                    befores.append(before)
+                if test.new_pidns:
+                    # Use the first BEFORE as the first process in the new pid namespace
+                    unshare_out = subprocess.Popen(["unshare", "--fork", "--pid", "--mount-proc", "-r", "--kill-child"] + ["--"] + test.befores[0].split(),
+                                                   start_new_session=True, universal_newlines=True,
+                                                   stdout=subprocess.PIPE,
+                                                   stderr=subprocess.STDOUT)
+
+                    lines = Runner.__wait_for_children(unshare_out.pid, test.timeout, "pid", lambda lines : len(lines) > 0)
+
+                    if not lines:
+                        raise TimeoutError(f'Timed out waiting create a new PID namespace')
+
+                    nsenter.extend(["nsenter", "-p", "-m", "-t", lines[0].strip()])
+
+                    # This is the only one we need to add to befores as killing the
+                    # unshare process will kill the whole process subtree with "--kill-child"
+                    befores.append(unshare_out)
+                    for before in test.befores[1:]:
+                        child_name = os.path.basename(before.strip().split()[0])[:15]
+                        before = subprocess.Popen(get_pid_ns_cmd(before),
+                                                start_new_session=True,
+                                                universal_newlines=True,
+                                                stdout=subprocess.PIPE,
+                                                stderr=subprocess.STDOUT)
+                        # wait for the child of nsenter
+                        if not Runner.__wait_for_children(before.pid, test.timeout, "comm", lambda lines : child_name in lines):
+                            raise TimeoutError(f'Timed out waiting for BEFORE {before}')
+                else:
+                    for before in test.befores:
+                        before = subprocess.Popen(before.split(),
+                                                start_new_session=True,
+                                                universal_newlines=True,
+                                                stdout=subprocess.PIPE,
+                                                stderr=subprocess.STDOUT)
+                        befores.append(before)
 
-                with open(os.devnull, 'w') as dn:
                     child_names = [os.path.basename(x.strip().split()[-1]) for x in test.befores]
                     child_names = sorted((x[:15] for x in child_names))  # cut to comm length
 
-                    # Print the names of all of our children and look
-                    # for the ones from BEFORE clauses
-                    waited=0
-                    while waited <= test.timeout:
-                        children = subprocess.run(["ps", "--ppid", str(os.getpid()), "--no-headers", "-o", "comm"],
-                                                  check=False, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
-                        if children.returncode == 0 and children.stdout:
-                            lines = [line.decode('utf-8') for line in children.stdout.splitlines()]
-                            lines = sorted((line.strip() for line in lines if line != 'ps'))
-                            if lines == child_names:
-                                break
-                        else:
-                            print(children.stderr)
+                    def found_all_children(lines):
+                        return sorted((line.strip() for line in lines if line != 'ps')) == child_names
 
-                        time.sleep(0.1)
-                        waited+=0.1
-
-                    if waited > test.timeout:
+                    if not Runner.__wait_for_children(os.getpid(), test.timeout, "comm", found_all_children):
                         raise TimeoutError(f'Timed out waiting for BEFORE(s) {test.befores}')
 
-            bpf_call = Runner.prepare_bpf_call(test)
+            bpf_call = Runner.prepare_bpf_call(test, nsenter)
             if test.befores and '{{BEFORE_PID}}' in bpf_call:
+                if test.new_pidns:
+                    # This can be fixed in the future if needed
+                    raise ValueError(f"BEFORE_PID cannot be used with NEW_PIDNS")
                 if len(test.befores) > 1:
                     raise ValueError(f"test has {len(test.befores)} BEFORE clauses but BEFORE_PID usage requires exactly one")
 
@@ -267,9 +338,12 @@ class Runner(object):
                 output += nextline
                 if not attached and nextline == "__BPFTRACE_NOTIFY_PROBES_ATTACHED\n":
                     attached = True
+                    if test.expect_mode != "regex":
+                        output = ""  # ignore earlier ouput
                     signal.alarm(test.timeout or DEFAULT_TIMEOUT)
                     if test.after:
-                        after = subprocess.Popen(test.after, shell=True,
+                        after_cmd = get_pid_ns_cmd(test.after) if test.new_pidns else test.after
+                        after = subprocess.Popen(after_cmd, shell=True,
                                                  start_new_session=True,
                                                  universal_newlines=True,
                                                  stdout=subprocess.PIPE,
@@ -277,7 +351,7 @@ class Runner(object):
 
             signal.alarm(0)
             output += p.stdout.read()
-            result = re.search(test.expect, output, re.M)
+            result = check_result(output)
 
         except (TimeoutError):
             # If bpftrace timed out (probably b/c the test case didn't explicitly
@@ -296,7 +370,7 @@ class Runner(object):
                 if p.poll() is None:
                     os.killpg(os.getpgid(p.pid), signal.SIGTERM)
                 output += p.stdout.read()
-                result = re.search(test.expect, output)
+                result = check_result(output)
 
                 if not result:
                     print(fail("[  TIMEOUT ] ") + "%s.%s" % (test.suite, test.name))
@@ -335,6 +409,9 @@ class Runner(object):
                 print('\tCLEANUP error: %s' % e.stderr)
                 return Runner.FAIL
 
+        def to_utf8(s):
+            return s.encode("unicode_escape").decode("utf-8")
+
         def print_befores_and_after_output():
             if len(befores_output) > 0:
                 for out in befores_output:
@@ -361,7 +438,22 @@ class Runner(object):
         else:
             print(fail("[  FAILED  ] ") + "%s.%s" % (test.suite, test.name))
             print('\tCommand: ' + bpf_call)
-            print('\tExpected: ' + test.expect)
-            print('\tFound: ' + output.encode("unicode_escape").decode("utf-8"))
+            if test.expect_mode == "regex":
+                print('\tExpected REGEX: ' + test.expect)
+                print('\tFound:\n' + to_utf8(output))
+            elif test.expect_mode == "json":
+                try:
+                    expected = json.dumps(json.loads(open(test.expect).read()), indent=2)
+                except json.decoder.JSONDecodeError as err:
+                    expected = "Could not parse JSON: " + str(err)
+                try:
+                    found = json.dumps(json.loads(output), indent=2)
+                except json.decoder.JSONDecodeError as err:
+                    found = "Could not parse JSON: " + str(err)
+                print('\tExpected JSON:\n' + expected)
+                print('\tFound:\n' + found)
+            else:
+                print('\tExpected FILE:\n\t\t' + to_utf8(open(test.expect).read()))
+                print('\tFound:\n\t\t' + to_utf8(output))
             print_befores_and_after_output()
             return Runner.FAIL
