@@ -189,9 +189,11 @@ TEST(semantic_analyser, builtin_variables)
   test("kprobe:f { probe }", 0);
   test("tracepoint:a:b { args }", 0);
   test("kprobe:f { fake }", 1);
+  test("kprobe:f { jiffies }", 0);
 
   MockBPFfeature feature(false);
   test(feature, "k:f { cgroup }", 1);
+  test(feature, "k:f { jiffies }", 1);
 }
 
 TEST(semantic_analyser, builtin_cpid)
@@ -221,6 +223,7 @@ TEST(semantic_analyser, builtin_functions)
   test("kprobe:f { @x = 1; print(@x) }", 0);
   test("kprobe:f { @x = 1; clear(@x) }", 0);
   test("kprobe:f { @x = 1; zero(@x) }", 0);
+  test("kprobe:f { @x = 1; @s = len(@x) }", 0);
   test("kprobe:f { time() }", 0);
   test("kprobe:f { exit() }", 0);
   test("kprobe:f { str(0xffff) }", 0);
@@ -385,6 +388,10 @@ TEST(semantic_analyser, compound_bxor)
 TEST(semantic_analyser, call_hist)
 {
   test("kprobe:f { @x = hist(1); }", 0);
+  test("kprobe:f { @x = hist(1, 0); }", 0);
+  test("kprobe:f { @x = hist(1, 5); }", 0);
+  test("kprobe:f { @x = hist(1, 10); }", 1);
+  test("kprobe:f { $n = 3; @x = hist(1, $n); }", 1);
   test("kprobe:f { @x = hist(); }", 1);
   test("kprobe:f { hist(1); }", 1);
   test("kprobe:f { $x = hist(1); }", 1);
@@ -588,6 +595,15 @@ TEST(semantic_analyser, call_zero)
   test("kprobe:f { @x = count(); zero(@x) ? 0 : 1; }", 10);
 }
 
+TEST(semantic_analyser, call_len)
+{
+  test("kprobe:f { @x[0] = 0; len(@x); }", 0);
+  test("kprobe:f { @x[0] = 0; len(); }", 1);
+  test("kprobe:f { @x[0] = 0; len(@x, 1); }", 1);
+  test("kprobe:f { @x[0] = 0; len(@x[2]); }", 1);
+  test("kprobe:f { $x = 0; len($x); }", 1);
+}
+
 TEST(semantic_analyser, call_time)
 {
   test("kprobe:f { time(); }", 0);
@@ -2822,6 +2838,14 @@ BEGIN { nsecs(xxx); }
 )");
 }
 
+TEST(semantic_analyser, config)
+{
+  test("config = { BPFTRACE_MAX_AST_NODES=1 } BEGIN { $ns = nsecs(); }", 0);
+  test("config = { BPFTRACE_MAX_AST_NODES=1; stack_mode=raw } BEGIN { $ns = "
+       "nsecs(); }",
+       0);
+}
+
 class semantic_analyser_btf : public test_btf
 {
 };
@@ -2895,6 +2919,31 @@ TEST_F(semantic_analyser_btf, iter)
   test("iter:task,f:func_1 { 1 }", 1);
 }
 
+// Sanity check for fentry/fexit aliases
+TEST_F(semantic_analyser_btf, fentry)
+{
+  test("fentry:func_1 { 1 }", 0);
+  test("fexit:func_1 { 1 }", 0);
+  test("fentry:func_1 { $x = args.a; $y = args.foo1; $z = args.foo2->f.a; }",
+       0);
+  test("fexit:func_1 { $x = retval; }", 0);
+  test("fentry:vmlinux:func_1 { 1 }", 0);
+  test("fentry:*:func_1 { 1 }", 0);
+  test("fentry:func_1 { @[func] = 1; }", 0);
+
+  test("fexit:func_1 { $x = args.foo; }", 1);
+  test("fexit:func_1 { $x = args; }", 0);
+  test("fentry:func_1 { @ = args; }", 0);
+  test("fentry:func_1 { @[args] = 1; }", 0);
+  // reg() is not available in fentry
+#ifdef ARCH_X86_64
+  test("fentry:func_1 { reg(\"ip\") }", 1);
+  test("fexit:func_1 { reg(\"ip\") }", 1);
+#endif
+  // Backwards compatibility
+  test("fentry:func_1 { $x = args->a; }", 0);
+}
+
 } // namespace semantic_analyser
 } // namespace test
 } // namespace bpftrace
