@@ -25,7 +25,7 @@ int main()
                      .d = { 5, 6, 7, 8 },
                      .e = { 9, 10, 11, 12 },
                      .f = { 13, 14, 15, 16 } };
-  strcpy(foo.a, "\x09\x08\x07\x06");
+  memcpy(foo.a, "\x09\x08\x07\x06", 4);
   func(&foo);
   free(foo.a);
   return 0;
