@@ -1,6 +1,7 @@
 struct Foo
 {
   int m;
+  int n;
 };
 
 int func(struct Foo *foo)
@@ -12,6 +13,7 @@ int main()
 {
   struct Foo foo;
   foo.m = 2;
+  foo.n = 3;
   func(&foo);
   return 0;
 }
