@@ -21,16 +21,8 @@ myclock() {
   return tv.tv_sec;
 }
 
-int
-main(int argc, char **argv __attribute__((unused))) {
-  if (argc > 1)
-  // If we don't have Systemtap headers, we should skip USDT tests. Returning 1 can be used as validation in the REQUIRE
-#ifndef HAVE_SYSTEMTAP_SYS_SDT_H
-    return 1;
-#else
-    return 0;
-#endif
-
+int main()
+{
   while (1) {
     myclock();
     // Sleep is necessary to not overflow perf buffer
