@@ -1,4 +1,4 @@
-#!/bin/bash
+#!/usr/bin/env bash
 
 set -u
 set +e;
@@ -17,7 +17,7 @@ IFS=',' read -ra SKIP_TOOLS <<< "${TOOLS_TEST_DISABLE:-"NONE"}"
 
 function set_tooldir() {
   local dir
-  for dir in "../../tools" "/vagrant/tools"; do
+  for dir in "${DIR}/../../tools" "/vagrant/tools"; do
       if [[ -d "$dir" ]]; then
           TOOLDIR="$dir"
           return
