@@ -4,19 +4,12 @@
 #include <cstring>
 #include <fcntl.h>
 #include <fstream>
+#include <stdlib.h>
 #include <sys/stat.h>
 #include <sys/types.h>
 #include <unistd.h>
 
-#if __has_include(<filesystem>)
-#include <filesystem>
-namespace std_filesystem = std::filesystem;
-#elif __has_include(<experimental/filesystem>)
-#include <experimental/filesystem>
-namespace std_filesystem = std::experimental::filesystem;
-#else
-#error "neither <filesystem> nor <experimental/filesystem> are present"
-#endif
+#include "filesystem.h"
 
 namespace bpftrace {
 namespace test {
@@ -265,6 +258,133 @@ TEST(utils, parse_kconfig)
   unlink(path);
 }
 
+TEST(utils, sanitiseBPFProgramName)
+{
+  const std::string name = "uprobe:/bin/bash:main+0x30";
+  const std::string sanitised = sanitise_bpf_program_name(name);
+  ASSERT_EQ(sanitised, "uprobe__bin_bash_main_0x30");
+
+  const std::string long_name =
+      "uretprobe:/this/is/a/very/long/path/to/a/binary/executable:"
+      "this_is_a_very_long_function_name_which_exceeds_the_KSYM_NAME_LEN_"
+      "limit_of_BPF_program_name";
+  const std::string long_sanitised = sanitise_bpf_program_name(long_name);
+  ASSERT_EQ(long_sanitised,
+            "uretprobe__this_is_a_very_long_path_to_a_binary_executable_this_"
+            "is_a_very_long_function_name_which_exceeds_the_ba30ddc67a52bad2");
+}
+
+// Run a function with environment var set to specific value.
+// Does its best to clean up env var so it doesn't leak between tests.
+static void with_env(const std::string &key,
+                     const std::string &val,
+                     std::function<void()> fn)
+{
+  EXPECT_EQ(::setenv(key.c_str(), val.c_str(), 1), 0);
+  try
+  {
+    fn();
+  }
+  catch (const std::exception &ex)
+  {
+    EXPECT_EQ(::unsetenv(key.c_str()), 0);
+    throw ex;
+  }
+  EXPECT_EQ(::unsetenv(key.c_str()), 0);
+}
+
+TEST(utils, find_in_path)
+{
+  std::string tmpdir = "/tmp/bpftrace-test-utils-XXXXXX";
+  ASSERT_TRUE(::mkdtemp(&tmpdir[0]));
+
+  // Create some directories
+  const std_filesystem::path path(tmpdir);
+  const std_filesystem::path usr_bin = path / "usr" / "bin";
+  const std_filesystem::path usr_local_bin = path / "usr" / "local" / "bin";
+  ASSERT_TRUE(std_filesystem::create_directories(usr_bin));
+  ASSERT_TRUE(std_filesystem::create_directories(usr_local_bin));
+
+  // Create some dummy binaries
+  const std_filesystem::path usr_bin_echo = usr_bin / "echo";
+  const std_filesystem::path usr_local_bin_echo = usr_local_bin / "echo";
+  const std_filesystem::path usr_bin_cat = usr_bin / "cat";
+  {
+    std::ofstream(usr_bin_echo) << "zz";
+    std::ofstream(usr_local_bin_echo) << "zz";
+    std::ofstream(usr_bin_cat) << "zz";
+  }
+
+  // Test basic find
+  with_env("PATH", usr_bin, [&]() {
+    auto f = find_in_path("echo");
+    ASSERT_TRUE(f.has_value());
+    EXPECT_TRUE(f->native().find("/usr/bin/echo") != std::string::npos);
+  });
+
+  // Test no entries found
+  with_env("PATH", usr_bin, [&]() {
+    auto f = find_in_path("echoz");
+    ASSERT_FALSE(f.has_value());
+  });
+
+  // Test precedence in find with two entries in $PATH
+  auto two_path = usr_local_bin.native() + ":" + usr_bin.native();
+  with_env("PATH", two_path, [&]() {
+    auto f = find_in_path("echo");
+    ASSERT_TRUE(f.has_value());
+    EXPECT_TRUE(f->native().find("/usr/local/bin/echo") != std::string::npos);
+  });
+
+  // Test no entries found with two entries in $PATH
+  with_env("PATH", two_path, [&]() {
+    auto f = find_in_path("echoz");
+    ASSERT_FALSE(f.has_value());
+  });
+
+  // Test empty $PATH
+  with_env("PATH", "", [&]() {
+    auto f = find_in_path("echo");
+    ASSERT_FALSE(f.has_value());
+  });
+
+  // Cleanup
+  EXPECT_TRUE(std_filesystem::remove_all(path));
+}
+
+TEST(utils, get_pids_for_program)
+{
+  auto pids = get_pids_for_program("/proc/self/exe");
+
+  ASSERT_EQ(pids.size(), 1);
+  ASSERT_EQ(pids[0], getpid());
+
+  pids = get_pids_for_program("/proc/12345/root/usr/bin/bash");
+  ASSERT_EQ(pids.size(), 0);
+}
+
+TEST(utils, file_exists_and_ownedby_root)
+{
+  std::string tmpdir = "/tmp/bpftrace-test-utils-XXXXXX";
+  std::string file1 = "/ownedby-user";
+  std::string file2 = "/no-exists";
+  if (::mkdtemp(tmpdir.data()) == nullptr)
+  {
+    throw std::runtime_error("creating temporary path for tests failed");
+  }
+
+  int fd;
+  fd = open((tmpdir + file1).c_str(), O_CREAT, S_IRUSR);
+  close(fd);
+  ASSERT_GE(fd, 0);
+
+  EXPECT_FALSE(file_exists_and_ownedby_root((tmpdir + file1).c_str()));
+  EXPECT_FALSE(file_exists_and_ownedby_root((tmpdir + file2).c_str()));
+  EXPECT_TRUE(file_exists_and_ownedby_root("/proc/1/maps"));
+
+  EXPECT_GT(std_filesystem::remove_all(tmpdir), 0);
+}
+
 } // namespace utils
 } // namespace test
 } // namespace bpftrace
